@@ -1,5 +1,5 @@
 /*
-    $Id: 64tass.h 2200 2020-04-07 19:18:23Z soci $
+    $Id: 64tass.h 2786 2022-05-25 04:08:01Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,27 +22,20 @@
 #include "stdbool.h"
 #include "inttypes.h"
 #include "wait_e.h"
-#ifndef REVISION
-#define REVISION "2200?"
-#endif
-#undef VERSION
-#define VERSION "1.55." REVISION
+
 #define MAX_PASS 20
 
 #define ignore() while(pline[lpoint.pos]==0x20 || pline[lpoint.pos]==0x09) lpoint.pos++
 #define here() pline[lpoint.pos]
 
 struct Obj;
-struct Listing;
 
-extern struct Listing *listing;
 extern address_t all_mem, all_mem2;
 extern unsigned int all_mem_bits;
 extern uint32_t outputeor;
-extern int temporary_label_branch;
-extern line_t vline;
+extern linenum_t vline;
 extern struct linepos_s lpoint;
-extern struct avltree *star_tree;
+extern struct star_s *star_tree;
 extern bool fixeddig, constcreated;
 extern address_t star;
 extern const uint8_t *pline;
