@@ -1,5 +1,5 @@
 /*
-    $Id: addressobj.c 2965 2023-01-08 19:50:05Z soci $
+    $Id: addressobj.c 3086 2023-09-03 06:23:08Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -171,6 +171,9 @@ static MUST_CHECK Obj *repr(Obj *o1, linepos_t epoint, size_t maxsize) {
         return NULL;
     }
 
+    if (ind != 0 && str->len != 0 && b2[ind - 1] == '+' && str->data[0] == '-') {
+        ind--; len--; chars--;
+    }
     v = new_str2(len);
     if (v == NULL) goto error;
     v->chars = chars;
@@ -532,7 +535,7 @@ static MUST_CHECK Obj *calc2(oper_t op) {
             if (am2 == A_NONE) break;
             if (op->op == O_ADD) {
                 return new_address(val_reference(v1->val), v1->type << 4 | am2);
-            } 
+            }
             if (op->op == O_SUB) {
                 atype_t am1 = A_NONE;
                 for (am = v1->type; ; am >>= 4) {
@@ -541,7 +544,7 @@ static MUST_CHECK Obj *calc2(oper_t op) {
                     case A_I:
                     case A_LI:
                     case A_IMMEDIATE:
-                    case A_IMMEDIATE_SIGNED: 
+                    case A_IMMEDIATE_SIGNED:
                         if (am2 == A_NONE) {
                             am1 = (am1 >> 4) | (amc << 12);
                             continue;
