@@ -1,5 +1,5 @@
 /*
-    $Id: arguments.c 2954 2023-01-07 10:22:44Z soci $
+    $Id: arguments.c 3115 2023-09-07 06:06:04Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,7 +36,6 @@ static const struct arguments_s arguments_default = {
     false,       /* to_ascii */
     false,       /* longbranch */
     false,       /* tasmcomp */
-    false,       /* make_phony */
     0x20,        /* caseinsensitive */
     NULL,        /* output */
     0,           /* output_len */
@@ -45,6 +44,7 @@ static const struct arguments_s arguments_default = {
     0,           /* symbol_output_len */
     NULL,        /* include */
     {            /* list */
+        {0,0,0}, /* name_pos */
         NULL,    /* name */
         true,    /* monitor */
         true,    /* source */
@@ -52,12 +52,22 @@ static const struct arguments_s arguments_default = {
         false,   /* verbose */
         false    /* append */
     },
-    NULL,        /* make */
-    {
+    {            /* make */
+        {0,0,0}, /* name_pos */
+        NULL,    /* name */
+        false,   /* phony */
+        false    /* append */
+    },
+    {            /* defines */
+        NULL,    /* data */
+        0,       /* len */
+    },
+    {            /* commandline */
         NULL,    /* data */
         0,       /* len */
     },
     {            /* error */
+        {0,0,0}, /* name_pos */
         NULL,    /* name */
         CARET_ALWAYS, /* caret */
         true,    /* warning */
@@ -102,10 +112,12 @@ static const struct diagnostics_s diagnostics_default = {
     true,        /* ignored */
     false,       /* long_branch */
     false,       /* altmode */
+    false,       /* align */
     true,        /* page */
     false,       /* macro_prefix */
     false,       /* float_round */
-    true         /* size_larger */
+    true,        /* size_larger */
+    true         /* priority */
 };
 
 static const struct diagnostics_s diagnostic_errors_default = {
@@ -143,10 +155,12 @@ static const struct diagnostics_s diagnostic_errors_default = {
     false,       /* ignored */
     false,       /* long_branch */
     false,       /* altmode */
+    false,       /* align */
     true,        /* page */
     false,       /* macro_prefix */
     false,       /* float_round */
-    false        /* size_larger */
+    false,       /* size_larger */
+    false        /* priority */
 };
 
 static struct diagnostics_s diagnostic_no_all;
@@ -187,10 +201,12 @@ static const struct diagnostics_s diagnostic_all_default = {
     true,        /* ignored */
     false,       /* long_branch */
     false,       /* altmode */
+    false,       /* align */
     true,        /* page */
     false,       /* macro_prefix */
     true,        /* float_round */
-    true         /* size_larger */
+    true,        /* size_larger */
+    true         /* priority */
 };
 
 static struct diagnostics_s diagnostic_no_error_all;
@@ -231,10 +247,12 @@ static const struct diagnostics_s diagnostic_error_all_default = {
     true,        /* ignored */
     true,        /* long_branch */
     true,        /* altmode */
+    true,        /* align */
     true,        /* page */
     true,        /* macro_prefix */
     true,        /* float_round */
-    true         /* size_larger */
+    true,        /* size_larger */
+    true         /* priority */
 };
 
 struct w_options_s {
@@ -275,10 +293,12 @@ static const struct w_options_s w_options[] = {
     {"ignored",         &diagnostics.ignored},
     {"long-branch",     &diagnostics.long_branch},
     {"altmode",         &diagnostics.altmode},
+    {"align",           &diagnostics.align},
     {"page",            &diagnostics.page},
     {"macro-prefix",    &diagnostics.macro_prefix},
     {"float-round",     &diagnostics.float_round},
     {"size-larger",     &diagnostics.size_larger},
+    {"priority",        &diagnostics.priority},
     {NULL,              NULL}
 };
 
@@ -347,7 +367,9 @@ enum {
     NO_CASE_SENSITIVE, NO_TASM_COMPATIBLE, NO_ASCII, CBM_PRG, S_RECORD,
     INTEL_HEX, APPLE_II, ATARI_XEX, MOS_HEX, NO_LONG_ADDRESS, NO_QUIET, WARN,
     OUTPUT_APPEND, NO_OUTPUT, ERROR_APPEND, NO_ERROR, LABELS_APPEND, MAP,
-    NO_MAP, MAP_APPEND, LIST_APPEND
+    NO_MAP, MAP_APPEND, LIST_APPEND, SIMPLE_LABELS, LABELS_SECTION,
+    MESEN_LABELS, LABELS_ADD_PREFIX, MAKE_APPEND, C256_PGX, C256_PGZ,
+    OUTPUT_EXEC
 };
 
 static const struct my_option long_options[] = {
@@ -366,6 +388,8 @@ static const struct my_option long_options[] = {
     {"mos-hex"          , my_no_argument      , NULL,  MOS_HEX},
     {"s-record"         , my_no_argument      , NULL,  S_RECORD},
     {"cbm-prg"          , my_no_argument      , NULL,  CBM_PRG},
+    {"c256-pgx"         , my_no_argument      , NULL,  C256_PGX},
+    {"c256-pgz"         , my_no_argument      , NULL,  C256_PGZ},
     {"no-ascii"         , my_no_argument      , NULL,  NO_ASCII},
     {"ascii"            , my_no_argument      , NULL, 'a'},
     {"no-tasm-compatible",my_no_argument      , NULL,  NO_TASM_COMPATIBLE},
@@ -390,6 +414,7 @@ static const struct my_option long_options[] = {
     {"no-output"        , my_no_argument      , NULL,  NO_OUTPUT},
     {"output-append"    , my_required_argument, NULL,  OUTPUT_APPEND},
     {"output-section"   , my_required_argument, NULL,  OUTPUT_SECTION},
+    {"output-exec"      , my_required_argument, NULL,  OUTPUT_EXEC},
     {"map"              , my_required_argument, NULL,  MAP},
     {"no-map"           , my_no_argument      , NULL,  NO_MAP},
     {"map-append"       , my_required_argument, NULL,  MAP_APPEND},
@@ -401,10 +426,15 @@ static const struct my_option long_options[] = {
     {"vice-labels"      , my_no_argument      , NULL,  VICE_LABELS},
     {"vice-labels-numeric",my_no_argument     , NULL,  VICE_LABELS_NUMERIC},
     {"dump-labels"      , my_no_argument      , NULL,  DUMP_LABELS},
+    {"simple-labels"    , my_no_argument      , NULL,  SIMPLE_LABELS},
+    {"mesen-labels"     , my_no_argument      , NULL,  MESEN_LABELS},
+    {"labels-add-prefix", my_required_argument, NULL,  LABELS_ADD_PREFIX},
     {"labels-root"      , my_required_argument, NULL,  LABELS_ROOT},
+    {"labels-section"   , my_required_argument, NULL,  LABELS_SECTION},
     {"list"             , my_required_argument, NULL, 'L'},
-    {"list-append"      , my_required_argument, NULL, LIST_APPEND},
+    {"list-append"      , my_required_argument, NULL,  LIST_APPEND},
     {"dependencies"     , my_required_argument, NULL, 'M'},
+    {"dependencies-append",my_required_argument,NULL,  MAKE_APPEND},
     {"no-make-phony"    , my_no_argument      , NULL,  NO_MAKE_PHONY},
     {"make-phony"       , my_no_argument      , NULL,  MAKE_PHONY},
     {"no-verbose-list"  , my_no_argument      , NULL,  NO_VERBOSE_LIST},
@@ -476,6 +506,8 @@ static address_t check_outputs(void) {
         case OUTPUT_IHEX:
         case OUTPUT_SREC:
         case OUTPUT_FLAT: min &= 0xffffffff; break;
+        case OUTPUT_PGX:
+        case OUTPUT_PGZ: min &= 0xffffff; break;
         case OUTPUT_MHEX:
         case OUTPUT_APPLE:
         case OUTPUT_XEX: min &= 0xffff; break;
@@ -485,7 +517,7 @@ static address_t check_outputs(void) {
         if (output->mapname != NULL && dash_name(output->mapname)) tostdout = true;
     }
     if (arguments.list.name != NULL && dash_name(arguments.list.name)) tostdout = true;
-    if (arguments.make != NULL && dash_name(arguments.make)) tostdout = true;
+    if (arguments.make.name != NULL && dash_name(arguments.make.name)) tostdout = true;
     if (!tostdout) {
         for (i = 0; i < arguments.symbol_output_len; i++) {
             struct symbol_output_s *symbol_output = &arguments.symbol_output[i];
@@ -502,7 +534,7 @@ static address_t check_outputs(void) {
 
 static struct include_list_s **include_list_add(struct include_list_s **lastil, const char *path)
 {
-    struct include_list_s *include; 
+    struct include_list_s *include;
     size_t i, j, len;
     j = i = strlen(path);
     if (i == 0) return lastil;
@@ -522,16 +554,48 @@ static struct include_list_s **include_list_add(struct include_list_s **lastil,
     return &include->next;
 }
 
-int testarg(int *argc2, char **argv2[]) {
+struct get_arg_s {
+    char **argv;
+    int optind;
+    uint32_t pos;
+};
+
+static void get_arg(struct get_arg_s *param, struct argpos_s *argpos) {
+    int i;
+    uint8_t *data;
+    uint32_t len = 0;
+    for (i = param->optind; i < my_optind; i++) {
+        if (inc_overflow(&len, strlen(param->argv[i]) + 1)) err_msg_out_of_memory();
+    }
+    argpos->start = param->pos;
+    argpos->line = (linenum_t)my_optind - 1;
+    argpos->pos = (linecpos_t)(len - strlen(my_optarg) - 1);
+    if (inc_overflow(&param->pos, len)) err_msg_out_of_memory();
+    if (param->pos > arguments.commandline.len) {
+        if (add_overflow(param->pos, 1024, &arguments.commandline.len)) err_msg_out_of_memory();
+        resize_array(&arguments.commandline.data, arguments.commandline.len);
+    }
+    data = arguments.commandline.data + (param->pos - len);
+    for (i = param->optind; i < my_optind; i++) {
+        size_t l = strlen(param->argv[i]);
+        memcpy(data, param->argv[i], l);
+        data += l;
+        *data++ = ' ';
+    }
+    data[-1] = 0;
+}
+
+int init_arguments(int *argc2, char **argv2[]) {
     int argc = *argc2;
     char **argv = *argv2;
     int opt;
     size_t defines_p = 0;
+    struct get_arg_s get_args;
     int max = 10;
     bool again;
     struct include_list_s **lastil = &arguments.include;
-    struct symbol_output_s symbol_output = { NULL, NULL, LABEL_64TASS, false };
-    struct output_s output = { "a.out", NULL, NULL, OUTPUT_CBM, false, false, false, false };
+    struct symbol_output_s symbol_output = { {0, 0, 0}, NULL, {0, 0, 0}, NULL, NULL, NULL, LABEL_64TASS, false };
+    struct output_s output = { {0, 0, 0}, "a.out", NULL, {0, 0, 0}, NULL, {0, 0, 0}, 0, OUTPUT_CBM, false, false, false, false };
     memcpy(&arguments, &arguments_default, sizeof arguments);
     memcpy(&diagnostics, &diagnostics_default, sizeof diagnostics);
     memcpy(&diagnostic_errors, &diagnostic_errors_default, sizeof diagnostic_errors);
@@ -540,11 +604,14 @@ int testarg(int *argc2, char **argv2[]) {
     memset(&diagnostic_no_error_all, 0, sizeof diagnostic_no_error_all);
     memcpy(&diagnostic_error_all, &diagnostic_error_all_default, sizeof diagnostic_error_all);
 
+    get_args.pos = 0;
     my_optind = 1;
     do {
         int i;
+        get_args.argv = argv;
         again = false;
         for (;;) {
+            get_args.optind = my_optind;
             opt = my_getopt_long(argc, argv, short_options, long_options, NULL);
             if (opt == -1) break;
             switch (opt) {
@@ -563,27 +630,36 @@ int testarg(int *argc2, char **argv2[]) {
             case INTEL_HEX:output.mode = OUTPUT_IHEX;break;
             case MOS_HEX:output.mode = OUTPUT_MHEX;break;
             case S_RECORD:output.mode = OUTPUT_SREC;break;
-            case CBM_PRG:output.mode = OUTPUT_CBM;break;
+            case CBM_PRG: output.mode = OUTPUT_CBM;break;
+            case C256_PGX: output.mode = OUTPUT_PGX;break;
+            case C256_PGZ: output.mode = OUTPUT_PGZ;break;
             case 'b':output.mode = OUTPUT_RAW;break;
             case 'f':output.mode = OUTPUT_FLAT;break;
             case 'a':arguments.to_ascii = true;break;
             case NO_ASCII:arguments.to_ascii = false;break;
             case 'T':arguments.tasmcomp = true;break;
             case NO_TASM_COMPATIBLE:arguments.tasmcomp = false;break;
-            case NO_OUTPUT:
+            case NO_OUTPUT: output.name = NULL; output.name_pos.start = 0; output.name_pos.line = 0; output.name_pos.pos = 0; output.append = false; break;
             case OUTPUT_APPEND:
-            case 'o': output.name = (opt == NO_OUTPUT) ? NULL : my_optarg;
-                      output.append = (opt == OUTPUT_APPEND) || (output.name != NULL && dash_name(output.name));
+            case 'o': output.name = my_optarg; get_arg(&get_args, &output.name_pos);
+                      output.append = (opt == OUTPUT_APPEND) || dash_name(output.name);
                       extend_array(&arguments.output, &arguments.output_len, 1);
                       arguments.output[arguments.output_len - 1] = output;
                       output.section = NULL;
                       output.mapname = NULL;
+                      output.mapname_pos.start = 0;
+                      output.mapname_pos.line = 0;
+                      output.mapname_pos.pos = 0;
                       output.mapfile = false;
+                      output.exec_pos.start = 0;
+                      output.exec_pos.line = 0;
+                      output.exec_pos.pos = 0;
                       break;
             case OUTPUT_SECTION:output.section = my_optarg; break;
+            case OUTPUT_EXEC: get_arg(&get_args, &output.exec_pos); break;
             case MAP_APPEND:
-            case MAP: output.mapname = my_optarg; output.mapappend = (opt == MAP_APPEND); output.mapfile = true; break;
-            case NO_MAP:output.mapname = NULL; output.mapfile = true; break;
+            case MAP: output.mapname = my_optarg; get_arg(&get_args, &output.mapname_pos); output.mapappend = (opt == MAP_APPEND); output.mapfile = true; break;
+            case NO_MAP:output.mapname = NULL; output.mapname_pos.start = 0; output.mapname_pos.line = 0; output.mapname_pos.pos = 0; output.mapfile = true; break;
             case CARET_DIAG:arguments.error.caret = CARET_ALWAYS;break;
             case MACRO_CARET_DIAG:arguments.error.caret = CARET_MACRO;break;
             case NO_CARET_DIAG:arguments.error.caret = CARET_NEVER;break;
@@ -614,22 +690,32 @@ int testarg(int *argc2, char **argv2[]) {
             case LABELS_APPEND:
             case 'l': symbol_output.name = my_optarg;
                       symbol_output.append = (opt == LABELS_APPEND);
+                      get_arg(&get_args, &symbol_output.name_pos);
                       extend_array(&arguments.symbol_output, &arguments.symbol_output_len, 1);
                       arguments.symbol_output[arguments.symbol_output_len - 1] = symbol_output;
-                      symbol_output.space = NULL;
+                      symbol_output.space_pos.start = 0;
+                      symbol_output.space_pos.line = 0;
+                      symbol_output.space_pos.pos = 0;
+                      symbol_output.section = NULL;
+                      symbol_output.add_prefix = NULL;
                       break;
             case NORMAL_LABELS: symbol_output.mode = LABEL_64TASS; break;
             case EXPORT_LABELS: symbol_output.mode = LABEL_EXPORT; break;
             case VICE_LABELS: symbol_output.mode = LABEL_VICE; break;
             case VICE_LABELS_NUMERIC: symbol_output.mode = LABEL_VICE_NUMERIC; break;
             case DUMP_LABELS: symbol_output.mode = LABEL_DUMP; break;
-            case LABELS_ROOT: symbol_output.space = my_optarg; break;
-            case NO_ERROR: arguments.error.name = NULL; arguments.error.no_output = true; arguments.error.append = false; break;
+            case SIMPLE_LABELS: symbol_output.mode = LABEL_SIMPLE; break;
+            case MESEN_LABELS: symbol_output.mode = LABEL_MESEN; break;
+            case LABELS_ROOT: get_arg(&get_args, &symbol_output.space_pos); break;
+            case LABELS_SECTION: symbol_output.section = my_optarg; break;
+            case LABELS_ADD_PREFIX: symbol_output.add_prefix = my_optarg; break;
+            case NO_ERROR: arguments.error.name = NULL; arguments.error.name_pos.start = 0; arguments.error.name_pos.line = 0; arguments.error.name_pos.pos = 0; arguments.error.no_output = true; arguments.error.append = false; break;
             case ERROR_APPEND:
-            case 'E': arguments.error.name = my_optarg; arguments.error.no_output = false; arguments.error.append = (opt == ERROR_APPEND); break;
+            case 'E': arguments.error.name = my_optarg; get_arg(&get_args, &arguments.error.name_pos); arguments.error.no_output = false; arguments.error.append = (opt == ERROR_APPEND); break;
             case LIST_APPEND:
-            case 'L': arguments.list.name = my_optarg; arguments.list.append = (opt == LIST_APPEND); break;
-            case 'M': arguments.make = my_optarg;break;
+            case 'L': arguments.list.name = my_optarg; get_arg(&get_args, &arguments.list.name_pos); arguments.list.append = (opt == LIST_APPEND); break;
+            case MAKE_APPEND:
+            case 'M': arguments.make.name = my_optarg; get_arg(&get_args, &arguments.make.name_pos); arguments.make.append = (opt == MAKE_APPEND); break;
             case 'I': lastil = include_list_add(lastil, my_optarg);break;
             case 'm': arguments.list.monitor = false;break;
             case MONITOR: arguments.list.monitor = true;break;
@@ -641,8 +727,8 @@ int testarg(int *argc2, char **argv2[]) {
             case NO_CASE_SENSITIVE: arguments.caseinsensitive = 0x20;break;
             case VERBOSE_LIST: arguments.list.verbose = true;break;
             case NO_VERBOSE_LIST: arguments.list.verbose = false;break;
-            case MAKE_PHONY: arguments.make_phony = true;break;
-            case NO_MAKE_PHONY: arguments.make_phony = false;break;
+            case MAKE_PHONY: arguments.make.phony = true;break;
+            case NO_MAKE_PHONY: arguments.make.phony = false;break;
             case TAB_SIZE:
                 {
                     char *s;
@@ -656,16 +742,21 @@ int testarg(int *argc2, char **argv2[]) {
                "        [-E <file>] [-I <path>] [-l <file>] [-L <file>] [-M <file>] [--ascii]\n"
                "        [--nostart] [--long-branch] [--case-sensitive] [--cbm-prg] [--flat]\n"
                "        [--atari-xex] [--apple-ii] [--intel-hex] [--mos-hex] [--s-record]\n"
-               "        [--nonlinear] [--tasm-compatible] [--quiet] [--no-warn] [--long-address]\n"
-               "        [--output-section=<name>] [--m65c02] [--m6502] [--m65xx] [--m65dtv02]\n"
-               "        [--m65816] [--m65el02] [--mr65c02] [--mw65c02] [--m65ce02] [--m4510]\n"
-               "        [--labels=<file>] [--normal-labels] [--export-labels] [--vice-labels]\n"
-               "        [--vice-labels-numeric] [--dump-labels] [--list=<file>]\n"
-               "        [--list-append=<file>] [--no-monitor] [--no-source] [--line-numbers]\n"
-               "        [--tab-size=<value>] [--verbose-list] [--dependencies=<file>]\n"
-               "        [--make-phony] [-W<option>] [--errors=<file>] [--output=<file>]\n"
-               "        [--output-append=<file>] [--no-output] [--map=<file>]\n"
-               "        [--map-append=<file>] [--no-map] [--help] [--usage] [--version] SOURCES\n");
+               "        [--nonlinear] [--c256-pgx] [--c256-pgz] [--tasm-compatible]\n"
+               "        [--long-address] [--output-section=<name>] [--m65c02] [--m6502]\n"
+               "        [--m65xx] [--m65dtv02] [--m65816] [--m65el02] [--mr65c02] [--mw65c02]\n"
+               "        [--m65ce02] [--m4510] [--labels=<file>] [--labels-append=<file>]\n"
+               "        [--labels-add-prefix=<txt>] [--labels-section=<name>]\n"
+               "        [--labels-root=<expr>] [--export-labels] [--vice-labels-numeric]\n"
+               "        [--vice-labels] [--dump-labels] [--simple-labels] [--mesen-labels]\n"
+               "        [--list=<file>] [--list-append=<file>] [--no-monitor] [--no-source]\n"
+               "        [--line-numbers] [--tab-size=<value>] [--verbose-list] [-W<option>]\n"
+               "        [--dependencies=<file>] [--dependencies-append=<file>] [--make-phony]\n"
+               "        [--output=<file>] [--output-append=<file>] [--output-exec=<expr>]\n"
+               "        [--no-output] [--map=<file>] [--map-append=<file>] [--no-map]\n"
+               "        [--error=<file>] [--error-append=<file>] [--quiet] [--no-warn]\n"
+               "        [--no-caret-diag] [--macro-caret-diag] [--help] [--usage] [--version]\n"
+               "        SOURCES\n");
                    return 0;
 
             case 'V':puts("64tass Turbo Assembler Macro V" VERSION);
@@ -684,6 +775,7 @@ int testarg(int *argc2, char **argv2[]) {
                "      --no-error         Do not output any errors\n"
                "  -I <path>              Include search path\n"
                "  -M, --dependencies=<f> Makefile dependencies to <file>\n"
+               "      --dependencies-append=<f> Append dependencies to <file>\n"
                "  -q, --quiet            Do not output summary and header\n"
                "  -T, --tasm-compatible  Enable TASM compatible mode\n"
                "  -w, --no-warn          Suppress warnings\n"
@@ -697,6 +789,7 @@ int testarg(int *argc2, char **argv2[]) {
                "  -Werror=<name>         Make a diagnostic to an error\n"
                "  -Wno-error=<name>      Make a diagnostic to a warning\n"
                "  -Walias                Warn about instruction aliases\n"
+               "  -Walign                Warn when alignment is done\n"
                "  -Waltmode              Warn about alternative addressing\n"
                "  -Wbranch-page          Warn if a branch crosses a page\n"
                "  -Wcase-symbol          Warn on mismatch of symbol case\n"
@@ -720,6 +813,7 @@ int testarg(int *argc2, char **argv2[]) {
                "  -Wno-wrap-pc           No PC bank cross warning\n"
                "  -Wno-pitfalls          No common pitfall notes\n"
                "  -Wno-portable          No portability warnings\n"
+               "  -Wno-priority          No operator priority warnings\n"
                "  -Wno-size-larger       No size larger than original warnings\n"
                "  -Wno-star-assign       No label multiply warnings\n"
                "  -Wold-equal            Warn about old equal operator\n"
@@ -736,6 +830,7 @@ int testarg(int *argc2, char **argv2[]) {
                "      --output-append=<f> Append output to <file>\n"
                "      --no-output        Do not create an output file\n"
                "      --output-section=<n> Output this section only\n"
+               "      --output-exec=<e>  Output execution address\n"
                "      --map=<f>          Place output map into <file>\n"
                "      --map-append=<f>   Append output map to <file>\n"
                "      --no-map           Do not create a map file\n"
@@ -749,6 +844,8 @@ int testarg(int *argc2, char **argv2[]) {
                "      --intel-hex        Output Intel HEX file\n"
                "      --mos-hex          Output MOS Technology file\n"
                "      --s-record         Output Motorola S-record file\n"
+               "      --c256-pgx         Output C256 PGX file\n"
+               "      --c256-pgz         Output C256 PGZ file\n"
                "\n"
                " Target CPU selection:\n"
                "      --m65xx            Standard 65xx (default)\n"
@@ -770,7 +867,10 @@ int testarg(int *argc2, char **argv2[]) {
                "      --vice-labels      Labels in VICE format\n"
                "      --vice-labels-numeric Labels for VICE with numeric constants\n"
                "      --dump-labels      Dump for debugging\n"
+               "      --simple-labels    Simple hexadecimal labels\n"
                "      --labels-root=<l>  List from scope <l> only\n"
+               "      --labels-section=<n> List from section <n> only\n"
+               "      --labels-add-prefix=<p> Set label prefix\n"
                "  -L, --list=<file>      List into <file>\n"
                "      --list-append=<f>  Append list to <file>\n"
                "  -m, --no-monitor       Don't put monitor code into listing\n"
@@ -840,13 +940,17 @@ int testarg(int *argc2, char **argv2[]) {
 
     if (arguments.symbol_output_len != 0) {
         arguments.symbol_output[arguments.symbol_output_len - 1].mode = symbol_output.mode;
-        if (symbol_output.space != NULL) arguments.symbol_output[arguments.symbol_output_len - 1].space = symbol_output.space;
+        if (symbol_output.space_pos.pos != 0) arguments.symbol_output[arguments.symbol_output_len - 1].space = symbol_output.space;
+        if (symbol_output.section != NULL) arguments.symbol_output[arguments.symbol_output_len - 1].section = symbol_output.section;
+        if (symbol_output.add_prefix != NULL) arguments.symbol_output[arguments.symbol_output_len - 1].add_prefix = symbol_output.add_prefix;
     }
 
     if (arguments.output == NULL) {
-        new_instance(&arguments.output);
-        arguments.output[0] = output;
-        arguments.output_len = 1;
+        if (output.name != NULL) {
+            new_instance(&arguments.output);
+            arguments.output[0] = output;
+            arguments.output_len = 1;
+        }
     } else {
         struct output_s *lastoutput = &arguments.output[arguments.output_len - 1];
         lastoutput->mode = output.mode;
@@ -866,7 +970,7 @@ int testarg(int *argc2, char **argv2[]) {
     if (defines_p != arguments.defines.len) {
         arguments.defines.len = defines_p;
         if (defines_p != 0) {
-            char *d = reallocate_array(arguments.defines.data, defines_p);
+            uint8_t *d = reallocate_array(arguments.defines.data, defines_p);
             if (d != NULL) arguments.defines.data = d;
         }
     }
@@ -883,6 +987,7 @@ void destroy_arguments(void) {
     free(arguments.output);
     free(arguments.symbol_output);
     free(arguments.defines.data);
+    free(arguments.commandline.data);
     include = arguments.include;
     while (include != NULL) {
         struct include_list_s *tmp = include;
