@@ -1,5 +1,5 @@
 /*
-    $Id: bitsobj.h 2651 2021-05-09 19:33:48Z soci $
+    $Id: bitsobj.h 3063 2023-08-27 09:19:30Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ struct Bytes;
 extern MUST_CHECK Obj *bits_calc1(Oper_types, unsigned int);
 extern MUST_CHECK Obj *bits_from_hexstr(const uint8_t *, linecpos_t *);
 extern MUST_CHECK Obj *bits_from_binstr(const uint8_t *, linecpos_t *);
-extern MUST_CHECK Obj *bits_from_str(const struct Str *, linepos_t);
+extern MUST_CHECK Obj *bits_from_str(struct Str *, linepos_t);
 extern MUST_CHECK Obj *bits_from_bytes(const struct Bytes *, linepos_t);
 extern MUST_CHECK Obj *bits_from_uval(uval_t, unsigned int);
 extern MUST_CHECK Obj *bits_from_obj(Obj *, linepos_t);
