@@ -1,5 +1,5 @@
 /*
-    $Id: boolobj.h 1944 2019-08-31 09:46:14Z soci $
+    $Id: boolobj.h 2676 2021-05-20 21:16:34Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,24 +28,31 @@ struct Str;
 typedef struct Bool {
     Obj v;
     struct Str *repr;
+    bool value;
 } Bool;
 
-extern Bool *true_value;
-extern Bool *false_value;
-extern Bool *bool_value[2];
+#define Bool(a) OBJ_CAST(Bool, a)
+
+extern Obj *const true_value;
+extern Obj *const false_value;
+extern Obj *const bool_value[2];
 
 extern void boolobj_init(void);
 extern void boolobj_names(void);
 extern void boolobj_destroy(void);
 
-static inline Bool *ref_bool(Bool *v1) {
-    v1->v.refcount++; return v1;
+static inline Obj *ref_true() {
+    true_value->refcount++; return true_value;
+}
+
+static inline Obj *ref_false() {
+    false_value->refcount++; return false_value;
 }
 
 static inline MUST_CHECK Obj *truth_reference(bool i) {
-    return (Obj *)ref_bool(bool_value[i ? 1 : 0]);
+    return val_reference(bool_value[i ? 1 : 0]);
 }
 
-extern MUST_CHECK struct Obj *int_from_bool(const struct Bool *);
-extern MUST_CHECK struct Float *float_from_bool(const struct Bool *);
+extern MUST_CHECK Obj *int_from_bool(const struct Bool *);
+extern MUST_CHECK Obj *float_from_bool(const struct Bool *);
 #endif
