@@ -1,5 +1,5 @@
 /*
-    $Id: encoding.h 2828 2022-10-20 20:29:55Z soci $
+    $Id: encoding.h 2981 2023-05-16 19:30:41Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,8 +24,7 @@ struct Enc;
 struct str_t;
 struct linepos_s;
 
-extern const char *const petscii_esc;
-#define identmap (const uint8_t *)petscii_esc
+extern const char *identmap;
 
 extern struct Enc *new_encoding(const struct str_t *, const struct linepos_s *);
 extern void init_encoding(bool);
