@@ -1,5 +1,5 @@
 /*
-    $Id: error.c 2956 2023-01-08 07:42:38Z soci $
+    $Id: error.c 3112 2023-09-06 06:34:22Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -42,6 +42,7 @@
 #include "console.h"
 
 struct file_list_s *current_file_list;
+struct file_list_s *commandline_file_list;
 const struct file_list_s *dummy_file_list;
 
 #define ALIGN(v) (((v) + (sizeof(int *) - 1)) & ~(sizeof(int *) - 1))
@@ -58,7 +59,7 @@ struct file_listnode_s {
 
 static struct file_listnode_s file_list;
 static const struct file_listnode_s *included_from = &file_list;
-static const char *prgname;
+static const char *prgname = "64tass";
 
 struct errorbuffer_s {
     size_t max;
@@ -338,6 +339,7 @@ static const char *const terr_warning[] = {
     "deprecated not equal operator, use '!=' instead",
     "deprecated directive, only for TASM compatible mode",
     "please use format(\"%d\", ...) as '^' will change it's meaning",
+    "please separate @b, @w or @l from label or number for future compatibility",
     "constant result, possibly changeable to 'lda'",
     "independent result, possibly changeable to 'lda'",
 #ifdef _WIN32
@@ -355,7 +357,7 @@ static const char *const terr_error[] = {
     "double defined range",
     "double defined escape",
     "extra characters on line",
-    "more than two characters",
+    "more than two characters ",
     "floating point overflow",
     "general syntax",
     "expression syntax",
@@ -375,12 +377,12 @@ static const char *const terr_error[] = {
     "empty range not allowed",
     "empty string not allowed",
     "empty list not allowed",
-    "more than a single character",
+    "more than a single character ",
     "requirements not met",
     "conflict",
     "index out of range ",
     "key not in dictionary ",
-    "offset out of range",
+    "offset out of range ",
     "not hashable ",
     "not a key and value pair ",
     "too large for a %u bit signed integer ",
@@ -388,7 +390,6 @@ static const char *const terr_error[] = {
     "too large for a %u byte signed integer ",
     "too large for a %u byte unsigned integer ",
     "value needs to be non-negative ",
-    "operands could not be broadcast together with shapes %" PRIuSIZE " and %" PRIuSIZE,
     "can't get sign of ",
     "can't get absolute value of ",
     "can't get integer value of ",
@@ -407,7 +408,8 @@ static const char *const terr_error[] = {
     "closing directive '",
     "opening directive '",
     "must be used within a loop",
-    "not measurable as start offset beyond size of original"
+    "not measurable as start offset beyond size of original",
+    "must be defined later"
 };
 
 static const char *const terr_fatal[] = {
@@ -489,6 +491,7 @@ void err_msg2(Error_types no, const void *prm, linepos_t epoint) {
         case ERROR_______OLD_NEQ:
         case ERROR____OLD_MODULO:
         case ERROR____OLD_STRING:
+        case ERROR________OLD_AT:
             new_error_msg2(diagnostic_errors.deprecated, epoint);
             adderror(terr_warning[no]);
             adderror(" [-Wdeprecated]");
@@ -561,6 +564,9 @@ void err_msg2(Error_types no, const void *prm, linepos_t epoint) {
         case ERROR____PTEXT_LONG:
             sprintf(line,"ptext too long by %" PRIuSIZE " bytes", *(const size_t *)prm - 0x100); adderror(line);
             break;
+        case ERROR____ALIGN_LONG:
+            sprintf(line,"block to long for alignment by %" PRIuaddress " bytes", *(const address_t *)prm); adderror(line);
+            break;
         case ERROR__BRANCH_CROSS:
             sprintf(line,"branch crosses page by %+d bytes", *(const int *)prm); adderror(line);
             break;
@@ -586,6 +592,10 @@ void err_msg2(Error_types no, const void *prm, linepos_t epoint) {
         case ERROR____NOT_DIRECT:
         case ERROR__NOT_DATABANK:
         case ERROR_CANT_CROSS_BA:
+        case ERROR__OFFSET_RANGE:
+        case ERROR_NOT_TWO_CHARS:
+        case ERROR__NOT_ONE_CHAR:
+        case ERROR______NOT_UVAL:
             adderror(terr_error[no - 0x40]);
             if (prm != NULL) err_msg_variable((Obj *)prm);
             break;
@@ -596,9 +606,9 @@ void err_msg2(Error_types no, const void *prm, linepos_t epoint) {
             break;
         case ERROR__NO_BYTE_ADDR:
         case ERROR__NO_WORD_ADDR:
-        case ERROR__NO_LONG_ADDR: 
-            adderror(terr_error[no - 0x40]); 
-            err_opcode(*(const uint32_t *)prm); 
+        case ERROR__NO_LONG_ADDR:
+            adderror(terr_error[no - 0x40]);
+            err_opcode(*(const uint32_t *)prm);
             break;
         default:
             adderror(terr_error[no - 0x40]);
@@ -614,11 +624,6 @@ void err_msg2(Error_types no, const void *prm, linepos_t epoint) {
         adderror2(((const str_t *)prm)->data, ((const str_t *)prm)->len);
         adderror("'");
         break;
-    case ERROR____LABEL_ROOT:
-        adderror("scope '");
-        adderror2(((const str_t *)prm)->data, ((const str_t *)prm)->len);
-        adderror("' for label listing not found");
-        break;
     case ERROR__SECTION_ROOT:
         adderror("section '");
         adderror2(((const str_t *)prm)->data, ((const str_t *)prm)->len);
@@ -641,6 +646,14 @@ static void err_msg_str_name(const char *msg, const str_t *name, linepos_t epoin
     if (more) new_error_msg_more();
 }
 
+void err_msg_enc_large(uval_t v, linepos_t epoint) {
+    char msg2[256];
+    bool more = new_error_msg(SV_ERROR, current_file_list, epoint);
+    sprintf(msg2, "encoded value %" PRIuval " larger than 8 bit", v);
+    adderror(msg2);
+    if (more) new_error_msg_more();
+}
+
 void err_msg_big_address(linepos_t epoint) {
     Obj *val = get_star_value(current_address->l_address, current_address->l_address_val);
     bool more = new_error_msg(SV_ERROR, current_file_list, epoint);
@@ -652,7 +665,7 @@ void err_msg_big_address(linepos_t epoint) {
 
 static bool err_msg_big_integer(const Error *err) {
     char msg2[256];
-    bool more = new_error_msg_err(err); 
+    bool more = new_error_msg_err(err);
     sprintf(msg2, terr_error[err->num - 0x40], err->u.intconv.bits);
     adderror(msg2);
     err_msg_variable(err->u.intconv.val);
@@ -803,7 +816,7 @@ static void err_msg_no_addressing(atype_t addrtype, uint32_t cod) {
 }
 
 static bool err_msg_no_register(const Error *err) {
-    bool more = new_error_msg_err(err); 
+    bool more = new_error_msg_err(err);
     Register *val = err->u.reg.reg;
     adderror("no register '");
     adderror2(val->data, val->len);
@@ -814,7 +827,7 @@ static bool err_msg_no_register(const Error *err) {
 
 static bool err_msg_no_lot_operand(const Error *err) {
     char msg2[256];
-    bool more = new_error_msg_err(err); 
+    bool more = new_error_msg_err(err);
     sprintf(msg2, "no %" PRIuSIZE " operand", err->u.opers.num);
     adderror(msg2);
     err_opcode(err->u.opers.cod);
@@ -823,8 +836,8 @@ static bool err_msg_no_lot_operand(const Error *err) {
 
 static bool err_msg_cant_broadcast(const Error *err) {
     char msg2[256];
-    bool more = new_error_msg_err(err); 
-    sprintf(msg2, terr_error[err->num - 0x40], err->u.broadcast.v1, err->u.broadcast.v2);
+    bool more = new_error_msg_err(err);
+    sprintf(msg2, "operands could not be broadcast together with shapes %" PRIuSIZE " and %" PRIuSIZE, err->u.broadcast.v1, err->u.broadcast.v2);
     adderror(msg2);
     return more;
 }
@@ -874,7 +887,7 @@ static void err_msg_argnum2(argcount_t num, argcount_t min, argcount_t max) {
     char line[1024];
     adderror("expected ");
     n = min;
-    if (min == max) { if (min != 0) adderror("exactly "); } 
+    if (min == max) { if (min != 0) adderror("exactly "); }
     else if (num < min) adderror("at least ");
     else {n = max; adderror("at most "); }
     switch (n) {
@@ -910,11 +923,9 @@ void err_msg_output(const Error *val) {
     case ERROR____CANT_IVAL2:
     case ERROR____CANT_UVAL2:
     case ERROR_____CANT_IVAL:
-    case ERROR_____CANT_UVAL:
-    case ERROR______NOT_UVAL: more = err_msg_big_integer(val); break;
+    case ERROR_____CANT_UVAL: more = err_msg_big_integer(val); break;
     case ERROR_REQUIREMENTS_:
     case ERROR______CONFLICT:
-    case ERROR_NOT_TWO_CHARS:
     case ERROR_NUMERIC_OVERF:
     case ERROR_NEGFRAC_POWER:
     case ERROR___EMPTY_RANGE:
@@ -922,7 +933,6 @@ void err_msg_output(const Error *val) {
     case ERROR____EMPTY_LIST:
     case ERROR__BYTES_NEEDED:
     case ERROR___NO_LAST_GAP:
-    case ERROR__NOT_ONE_CHAR:
     case ERROR_NO_ZERO_VALUE:
     case ERROR_OUT_OF_MEMORY:
     case ERROR__ADDR_COMPLEX:
@@ -934,6 +944,8 @@ void err_msg_output(const Error *val) {
     case ERROR__NO_BYTE_ADDR:
     case ERROR__NO_WORD_ADDR:
     case ERROR__NO_LONG_ADDR: more = new_error_msg_err(val); adderror(terr_error[val->num - 0x40]); err_opcode(val->u.addresssize.cod); break;
+    case ERROR______NOT_UVAL:
+    case ERROR__NOT_ONE_CHAR:
     case ERROR_DIVISION_BY_Z:
     case ERROR_ZERO_NEGPOWER:
     case ERROR__NOT_KEYVALUE:
@@ -1001,11 +1013,17 @@ void err_msg_still_none(const str_t *name, linepos_t epoint) {
     if (name != NULL) str_name(name->data, name->len);
 }
 
+void err_msg_still_align(linepos_t epoint) {
+    bool more = new_error_msg(SV_ERROR, current_file_list, epoint);
+    adderror("can't do the alignment of this");
+    if (more) new_error_msg_more();
+}
+
 void err_msg_not_defined(const str_t *name, linepos_t epoint) {
     err_msg_str_name("not defined", name, epoint);
 }
 
-unsigned int err_msg_unknown_formatchar(const Str *s, size_t offs, linepos_t epoint) {
+void err_msg_unknown_formatchar(const Str *s, size_t offs, linepos_t epoint) {
     struct linepos_s epoint2 = *epoint;
     unsigned int len;
     bool more;
@@ -1020,7 +1038,6 @@ unsigned int err_msg_unknown_formatchar(const Str *s, size_t offs, linepos_t epo
         adderror("'");
     }
     if (more) new_error_msg_more();
-    return len;
 }
 
 static void err_msg_double_note(const struct file_list_s *cflist, linepos_t epoint, const str_t *labelname2) {
@@ -1324,13 +1341,41 @@ void err_msg_branch_page(int by, linepos_t epoint) {
     adderror(msg2);
 }
 
-void err_msg_page(address_t adr, address_t adr2, linepos_t epoint) {
+void err_msg_align(address_t by, linepos_t epoint) {
+    char msg2[256];
+    new_error_msg2(diagnostic_errors.align, epoint);
+    sprintf(msg2, "aligned by %" PRIuaddress " bytes [-Walign]", by);
+    adderror(msg2);
+}
+
+void err_msg_alignblk(address_t by, address_t by2, linepos_t epoint) {
+    char msg2[256];
+    new_error_msg2(diagnostic_errors.align, epoint);
+    sprintf(msg2, "over the boundary by %" PRIuaddress " bytes, aligned by %" PRIuaddress " bytes [-Walign]", by, by2);
+    adderror(msg2);
+}
+
+void err_msg_page(address_t adr, address_t adr2, uval_t size, linepos_t epoint) {
     char line[256];
     new_error_msg2(diagnostic_errors.page, epoint);
-    sprintf(line,"different start and end page $%04" PRIxaddress " and $%04" PRIxaddress " [-Wpage]", adr, adr2);
+    sprintf(line,"start $%04" PRIxaddress " not on same %" PRIuval " byte page as end $%04" PRIxaddress " [-Wpage]", adr, size, adr2);
     adderror(line);
 }
 
+void err_msg_page_cross(address_t at, address_t by, uval_t size, linepos_t epoint) {
+    char line[256];
+    new_error_msg2(diagnostic_errors.page, epoint);
+    sprintf(line,"crossing of %" PRIuval " byte page at $%04" PRIxaddress " by %" PRIuaddress " bytes [-Wpage]", size, at, by);
+    adderror(line);
+}
+
+void err_msg_priority(const Oper *op, linepos_t epoint) {
+    new_error_msg2(diagnostic_errors.priority, epoint);
+    adderror("parentheses suggested as ");
+    adderror(op->name);
+    adderror("' applies to the whole expression [-Wpriority]");
+}
+
 void err_msg_alias(uint32_t a, uint32_t b, linepos_t epoint) {
     char name[4];
     new_error_msg2(diagnostic_errors.alias, epoint);
@@ -1355,7 +1400,7 @@ static void err_unicode_character(unichar_t ch) {
     *s++ = quote;
     if (ch != 0 && ch < 0x80) *s++ = (uint8_t)ch; else s += utf8out(ch, s);
     *s++ = quote;
-    sprintf((char *)s, " (U+%04" PRIX32 ")", ch); 
+    sprintf((char *)s, " (U+%04" PRIX32 ")", ch);
     adderror((char *)line);
 }
 
@@ -1385,7 +1430,7 @@ void err_msg_unknown_char(unichar_t ch, linepos_t epoint) {
     if (more) new_error_msg_more();
 }
 
-bool err_msg_wrong_character(linepos_t epoint) {
+void err_msg_wrong_character(linepos_t epoint) {
     unichar_t ch, ch2 = pline[epoint->pos];
     if ((ch2 & 0x80) != 0) utf8in(pline + epoint->pos, &ch2);
     switch (ch2) {
@@ -1401,11 +1446,13 @@ bool err_msg_wrong_character(linepos_t epoint) {
     case 0x201C: case 0x201D: case 0x201E: case 0x201F:
         ch = '"';
         break;
+    case 0x2217:
+        ch = '*';
+        break;
     default:
         ch = 0;
     }
     err_msg_wrong_character2(ch, ch2, epoint);
-    return true;
 }
 
 static const uint8_t *printline(const struct file_list_s *cfile, linepos_t epoint, const uint8_t *line, FILE *f) {
@@ -1460,7 +1507,7 @@ static void print_error(FILE *f, const struct errorentry_s *err, bool caret) {
     }
     if (console_use_color) {
         if (bold) {
-            console_defaultbold(f); 
+            console_defaultbold(f);
 #ifdef COLOR_OUTPUT
             console_use_bold = true;
 #endif
@@ -1518,7 +1565,7 @@ void error_print(const struct error_output_s *output) {
     if (output->name != NULL) {
         ferr = dash_name(output->name) ? stdout : fopen_utf8(output->name, output->append ? "at" : "wt");
         if (ferr == NULL) {
-            err_msg_file2(ERROR_CANT_WRTE_ERR, output->name);
+            err_msg_file2(ERROR_CANT_WRTE_ERR, output->name, &output->name_pos);
             ferr = stderr;
         }
     } else ferr = output->no_output ? NULL : stderr;
@@ -1621,6 +1668,7 @@ void err_init(const char *name) {
     avltree_init(&error_list.members);
     current_file_list = &file_list.flist;
     dummy_file_list = &file_list.flist;
+    commandline_file_list = &file_list.flist;
     included_from = &file_list;
     avltree_init(&notdefines);
     lastnd = NULL;
@@ -1645,7 +1693,7 @@ void fatal_error(const char *txt) {
     if (txt != NULL) {
         console_use(stderr);
         if (console_use_color) console_bold(stderr);
-        printable_print((const uint8_t *)((prgname != NULL) ? prgname : "64tass"), stderr);
+        printable_print((const uint8_t *)prgname, stderr);
         fputs(": ", stderr);
         if (console_use_color) console_red(stderr);
         fputs("fatal error: ", stderr);
@@ -1689,9 +1737,10 @@ void err_msg_file(Error_types no, const char *prm, const struct file_list_s *cfi
     if (more) new_error_msg_more();
 }
 
-void err_msg_file2(Error_types no, const char *prm) {
-    static const struct linepos_s nopoint = {0, 0};
-    err_msg_file(no, prm, current_file_list, &nopoint);
+void err_msg_file2(Error_types no, const char *name, const struct argpos_s *prm) {
+    pline = arguments.commandline.data + prm->start;
+    lpoint.pos = prm->pos; lpoint.line = prm->line;
+    err_msg_file(no, name, commandline_file_list, &lpoint);
 }
 
 void error_status(void) {
