@@ -1,5 +1,5 @@
 /*
-    $Id: errorobj.h 2079 2019-11-11 20:40:59Z soci $
+    $Id: errorobj.h 2808 2022-10-17 04:49:11Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #define ERROROBJ_H
 #include "obj.h"
 #include "errors_e.h"
-#include "str.h"
+#include "oper_e.h"
 #include "stdbool.h"
 
 extern struct Type *const ERROR_OBJ;
@@ -34,12 +34,12 @@ typedef struct Error {
     const uint8_t *line;
     union {
         struct {
-            const struct Oper *op;
+            Oper_types op;
             Obj *v1;
             Obj *v2;
         } invoper;
         struct {
-            Obj *ident;
+            Obj *symbol;
             struct Namespace *names;
             bool down;
         } notdef;
@@ -60,27 +60,35 @@ typedef struct Error {
             uint32_t cod;
         } reg;
         struct {
-            size_t num, cod;
+            size_t num;
+            uint32_t cod;
         } opers;
         struct {
-            size_t cod;
+            uint32_t cod;
         } addresssize;
         struct {
             size_t v1;
             size_t v2;
         } broadcast;
         struct {
-            size_t num, min, max;
+            argcount_t num, min, max;
         } argnum;
+        struct {
+            const struct Type *t1;
+            const struct Type *t2;
+        } otype;
     } u;
 } Error;
 
+#define Error(a) OBJ_CAST(Error, a)
+
 extern void errorobj_init(void);
 
 extern MALLOC Error *new_error(Error_types, linepos_t);
-extern MALLOC Error *new_error_mem(linepos_t);
-extern MALLOC Error *new_error_obj(Error_types, Obj *, linepos_t);
-extern MALLOC Error *new_error_conv(Obj *, struct Type *, linepos_t);
-extern MALLOC Error *new_error_argnum(size_t, size_t, size_t, linepos_t);
+extern MALLOC Obj *new_error_mem(linepos_t);
+extern MALLOC Obj *new_error_obj(Error_types, Obj *, linepos_t);
+extern MALLOC Obj *new_error_conv(Obj *, struct Type *, linepos_t);
+extern MALLOC Obj *new_error_argnum(argcount_t, argcount_t, argcount_t, linepos_t);
+extern void error_obj_update(Error *, const Obj *, Obj *);
 
 #endif
