@@ -1,5 +1,5 @@
 /*
-    $Id: eval.c 2892 2022-11-04 07:35:54Z soci $
+    $Id: eval.c 3114 2023-09-06 20:55:15Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -371,10 +371,10 @@ static MUST_CHECK Obj *resolv_anonlabel(ssize_t as, linecpos_t pos) {
     if (l != NULL) {
         touch_label(l);
         return val_reference(l->value);
-    } 
+    }
     if (constcreated && pass < max_pass) {
         return ref_none();
-    } 
+    }
     epoint.line = lpoint.line;
     epoint.pos = pos;
     err = new_error(ERROR___NOT_DEFINED, &epoint);
@@ -499,10 +499,8 @@ rest:
             llen = (linecpos_t)get_label(pline + lpoint.pos);
             if (llen == 0) {
                 if (opr.p != opr.data) epoint.pos = opr.p[-1].pos;
-                if ((ch < ' ' && ch != 0) || ch > '~') {
-                    if (err_msg_wrong_character(&lpoint)) goto error;
-                }
-                err_msg2(ERROR______EXPECTED, "an expression is", &lpoint);
+                if ((ch < ' ' && ch != 0) || ch > '~') err_msg_wrong_character(&lpoint);
+                else err_msg2(ERROR______EXPECTED, "an expression is", &lpoint);
                 goto error;
             }
             lpoint.pos += llen;
@@ -549,12 +547,12 @@ rest:
                 switch (pline[epoint.pos + 1] | arguments.caseinsensitive) {
                 case 'x':
                     opr.p->op = O_COMMAX;
-                    opr.p->pos = epoint.pos; 
+                    opr.p->pos = epoint.pos;
                     if (++opr.p == opr.max) extend_opr(&opr);
                     goto other;
                 case 'y':
                     opr.p->op = O_COMMAY;
-                    opr.p->pos = epoint.pos; 
+                    opr.p->pos = epoint.pos;
                     if (++opr.p == opr.max) extend_opr(&opr);
                     goto other;
                 default: break;
@@ -584,10 +582,10 @@ rest:
         case '/': op = O_DIV; goto add;
         case '+': op = O_ADD; goto add;
         case '-': op = O_SUB;
-        add: 
+        add:
             lpoint.pos++;
             opr.p->op = op;
-            opr.p->pos = epoint.pos; 
+            opr.p->pos = epoint.pos;
             if (++opr.p == opr.max) extend_opr(&opr);
             continue;
         case ')':
@@ -617,10 +615,8 @@ rest:
             }
             break;
         default:
-            if (ch < ' ' || ch > '~') {
-                if (err_msg_wrong_character(&epoint)) goto error;
-            }
-            err_msg2(ERROR______EXPECTED, "an operator is", &epoint);
+            if (ch < ' ' || ch > '~') err_msg_wrong_character(&epoint);
+            else err_msg2(ERROR______EXPECTED, "an operator is", &epoint);
             goto error;
         }
         if (opr.p == opr.data) {
@@ -814,7 +810,7 @@ static bool get_val2_compat(struct eval_context_s *ev) {/* length in bytes, defi
                     v[0].val = int_from_uval(val1);
                     continue;
                 }
-            default: 
+            default:
                 epoint.pos = out->pos;
                 break;
             case T_ERROR:
@@ -962,22 +958,25 @@ static bool get_val2(struct eval_context_s *ev) {
                     v--;
                 }
                 args -= vsp;
-                if (args == 1) {
-                    if (stop && !expc) {
-                        if (tup && out + 1 != ev->out.end) {
-                            Obj *obj = out[1].val;
-                            if (obj->obj != OPER_OBJ ||
-                                    (Oper(obj)->op != O_RPARENT &&   /* ((3)) */
-                                     Oper(obj)->op != O_RBRACKET &&  /* [(3)] */
-                                     Oper(obj)->op != O_FUNC &&      /* f((3)) */
-                                     Oper(obj)->op != O_LIST &&      /* [(3),] */
-                                     Oper(obj)->op != O_COMMA &&     /* [(3),(3)] */
-                                     !(Oper(obj)->op >= O_COMMAX && Oper(obj)->op <= O_COMMAK) /* (3),y */
-                                    )) {
-                                v[0].val = v[1].val; v[1].val = NULL;
-                                continue;
-                            }
+                if (stop && !expc) {
+                    Address_types am2;
+                    if (out + 1 != ev->out.end) {
+                        Obj *obj = out[1].val;
+                        if (obj->obj != OPER_OBJ ||
+                            (Oper(obj)->op != O_RPARENT &&   /* ((3)) */
+                             Oper(obj)->op != O_RBRACKET &&  /* [(3)] */
+                             Oper(obj)->op != O_RBRACE &&    /* {(3)} */
+                             Oper(obj)->op != O_FUNC &&      /* f((3)) */
+                             Oper(obj)->op != O_INDEX &&     /* m[(3)] */
+                             Oper(obj)->op != O_COMMA &&     /* [(3),(3)] */
+                             !(Oper(obj)->op >= O_COMMAX && Oper(obj)->op <= O_COMMAK) /* (3),y */
+                            )) {
+                            goto nind;
                         }
+                    }
+                    switch (args) {
+                    case 1:
+                    ind1:
                         am = (op == O_BRACKET) ? A_LI : A_I;
                         if (v[1].val->obj != ADDRESS_OBJ && !v[1].val->obj->iterable) {
                             v[0].val = new_address(v[1].val, am);
@@ -987,29 +986,53 @@ static bool get_val2(struct eval_context_s *ev) {
                         }
                         v[1].val = NULL;
                         continue;
+                    case 2:
+                        if (v[2].val->obj != REGISTER_OBJ || Register(v[2].val)->len != 1) goto nind2;
+                        am = register_to_indexing(Register(v[2].val)->data[0]);
+                        if (am == A_NONE) goto nind2;
+                    ind2:
+                        val_destroy(v[2].val);
+                        v[2].val = NULL;
+                        if (v[1].val->obj != ADDRESS_OBJ && !v[1].val->obj->iterable) {
+                            v[1].val = new_address(v[1].val, am);
+                        } else {
+                            Obj *tmp = apply_addressing(v[1].val, am, true);
+                            val_destroy(v[1].val);
+                            v[1].val = tmp;
+                        }
+                        goto ind1;
+                    case 3:
+                        if (v[3].val->obj != REGISTER_OBJ || Register(v[3].val)->len != 1) goto nind2;
+                        am2 = register_to_indexing(Register(v[3].val)->data[0]);
+                        if (am2 == A_NONE) goto nind2;
+                        if (v[2].val->obj != REGISTER_OBJ || Register(v[2].val)->len != 1) goto nind2;
+                        am = register_to_indexing(Register(v[2].val)->data[0]);
+                        if (am == A_NONE) goto nind2;
+                        val_destroy(v[2].val);
+                        v[2].val = v[3].val;
+                        v[3].val = NULL;
+                        if (v[1].val->obj != ADDRESS_OBJ && !v[1].val->obj->iterable) {
+                            v[1].val = new_address(v[1].val, am);
+                        } else {
+                            Obj *tmp = apply_addressing(v[1].val, am, true);
+                            val_destroy(v[1].val);
+                            v[1].val = tmp;
+                        }
+                        am = am2;
+                        goto ind2;
+                    default: break;
                     }
+                }
+            nind:
+                if (args == 1) {
                     if (tup) {
                         v[0].val = v[1].val; v[1].val = NULL;
                         v[0].epoint.pos = v[1].epoint.pos;
                         continue;
                     }
                 }
-                if (args == 2 && stop && !expc) {
-                    if (out + 1 == ev->out.end && v[2].val->obj == REGISTER_OBJ && Register(v[2].val)->len == 1) {
-                        atype_t addrtype = (op == O_BRACKET) ? A_LI: A_I;
-                        addrtype |= register_to_indexing(Register(v[2].val)->data[0]) << 4;
-                        val_destroy(v[2].val);
-                        if (v[1].val->obj != ADDRESS_OBJ && !v[1].val->obj->iterable) {
-                            v[0].val = new_address(v[1].val, addrtype);
-                        } else {
-                            v[0].val = apply_addressing(v[1].val, addrtype, true);
-                            val_destroy(v[1].val);
-                        }
-                        v[1].val = NULL;
-                        continue;
-                    }
-                }
                 if (args != 0) {
+            nind2:
                     list = List(val_alloc((op == O_BRACKET) ? LIST_OBJ : TUPLE_OBJ));
                     list->len = args;
                     list->data = list_create_elements(list, args);
@@ -1079,7 +1102,7 @@ static bool get_val2(struct eval_context_s *ev) {
                 tmp.v.obj = FUNCARGS_OBJ;
 
                 epoint.pos = out->pos;
-                oper.v1 = none_value; 
+                oper.v1 = none_value;
                 oper.v2 = &tmp.v;
                 oper.epoint = &epoint;
                 oper.epoint2 = &v[0].epoint;
@@ -1223,8 +1246,8 @@ static bool get_val2(struct eval_context_s *ev) {
                 continue;
             }
             continue;
-        case O_IDENTITY: 
-        case O_NIDENTITY: 
+        case O_IDENTITY:
+        case O_NIDENTITY:
             if (vsp < 2) goto syntaxe;
             vsp--;
             v = &values[vsp - 1];
@@ -1431,15 +1454,28 @@ static bool get_exp2(int stop) {
         case '*': op = O_SPLAT; break;
         case '!': op = O_LNOT;break;
         case '~': op = O_INV; break;
-        case '<': if (pline[lpoint.pos + 1] == '>') {lpoint.pos++;op = O_WORD;} else op = O_LOWER; break;
-        case '>': if (pline[lpoint.pos + 1] == '`') {lpoint.pos++;op = O_HWORD;} else if (pline[lpoint.pos + 1] == '<') {lpoint.pos++;op = O_BSWORD;} else op = O_HIGHER; break;
-        case '#': op = (pline[lpoint.pos + 1] == '+' || pline[lpoint.pos + 1] == '-') ? O_HASH_SIGNED : O_HASH; break;
-        case '`': op = O_BANK; break;
-        case '^': op = O_STRING; if (diagnostics.deprecated) err_msg2(ERROR____OLD_STRING, NULL, &lpoint); break;
+        case '<': if (pline[lpoint.pos + 1] == '>') {lpoint.pos++;op = O_WORD;} else op = O_LOWER; goto priocheck;
+        case '>': if (pline[lpoint.pos + 1] == '`') {lpoint.pos++;op = O_HWORD;} else if (pline[lpoint.pos + 1] == '<') {lpoint.pos++;op = O_BSWORD;} else op = O_HIGHER; goto priocheck;
+        case '#': op = (pline[lpoint.pos + 1] == '+' || pline[lpoint.pos + 1] == '-') ? O_HASH_SIGNED : O_HASH; goto priocheck;
+        case '`': op = O_BANK; goto priocheck;
+        case '^':
+            op = O_STRING;
+            if (diagnostics.deprecated) err_msg2(ERROR____OLD_STRING, NULL, &lpoint);
+        priocheck:
+            if (diagnostics.priority && opr.p != opr.data && opr.p - 1 != opr.data && op == opr.p[-2].op) {
+                unsigned int prio = operators[opr.p[-1].op].prio;
+                if (prio > operators[op].prio && prio != operators[O_NEG].prio && prio != operators[O_SPLAT].prio) {
+                    struct linepos_s epoint2;
+                    epoint2.line = lpoint.line;
+                    epoint2.pos = opr.p[-2].pos;
+                    err_msg_priority(&operators[op], &epoint2);
+                }
+            }
+            break;
         case '$': val = get_hex(&epoint); goto push_other;
-        case '%': 
-            if ((pline[lpoint.pos + 1] & 0xfe) == 0x30 || (pline[lpoint.pos + 1] == '.' && (pline[lpoint.pos + 2] & 0xfe) == 0x30)) { 
-                val = get_bin(&epoint); goto push_other; 
+        case '%':
+            if ((pline[lpoint.pos + 1] & 0xfe) == 0x30 || (pline[lpoint.pos + 1] == '.' && (pline[lpoint.pos + 2] & 0xfe) == 0x30)) {
+                val = get_bin(&epoint); goto push_other;
             }
             goto tryanon;
         case '"':
@@ -1474,17 +1510,17 @@ static bool get_exp2(int stop) {
                     val = new_symbol(&symbol, &epoint);
                     goto push_other;
                 case '(':
-                    lpoint.pos++; 
-                    symbollist++; 
+                    lpoint.pos++;
+                    symbollist++;
                     goto tphack2;
                 case '[':
-                    lpoint.pos++; 
-                    symbollist++; 
+                    lpoint.pos++;
+                    symbollist++;
                     goto lshack2;
                 case '.':
                     if (symbol.data[1] == '.') {
-                        lpoint.pos += 3; 
-                        val = ref_fold(); 
+                        lpoint.pos += 3;
+                        val = ref_fold();
                         goto push_other;
                     }
                     FALL_THROUGH; /* fall through */
@@ -1600,10 +1636,8 @@ static bool get_exp2(int stop) {
                 }
                 epoint.pos = opr.p[-1].pos;
             }
-            if ((ch < ' ' && ch != 0) || ch > '~') {
-                if (err_msg_wrong_character(&lpoint)) goto error;
-            }
-            err_msg2(ERROR______EXPECTED, "an expression is", &lpoint);
+            if ((ch < ' ' && ch != 0) || ch > '~') err_msg_wrong_character(&lpoint);
+            else err_msg2(ERROR______EXPECTED, "an expression is", &lpoint);
             goto error;
         }
         if (opr.p != opr.data && opr.p[-1].op == O_SPLAT) {
@@ -1663,7 +1697,7 @@ static bool get_exp2(int stop) {
                         opr.p->pos = epoint.pos;
                         if (++opr.p == opr.max) extend_opr(&opr);
                         goto other;
-                    } 
+                    }
                 }
             } else llen = 0;
             prec = operators[O_COMMA].prio;
@@ -1776,7 +1810,7 @@ static bool get_exp2(int stop) {
                 opr.p->pos = epoint.pos;
                 if (++opr.p == opr.max) extend_opr(&opr);
                 continue;
-            } 
+            }
             op = pline[lpoint.pos + 2] != '=' ? O_EQ : O_IDENTITY;
         push2:
             lpoint.pos += operators[op].len;
@@ -1815,7 +1849,7 @@ static bool get_exp2(int stop) {
             goto push2;
         case '!':
             if (pline[lpoint.pos + 1] == '=') {op = pline[lpoint.pos + 2] != '=' ? O_NE : O_NIDENTITY;goto push2;}
-            if (get_label(pline + lpoint.pos + 1) == 2 && 
+            if (get_label(pline + lpoint.pos + 1) == 2 &&
                 (pline[epoint.pos + 1] | arguments.caseinsensitive) == 'i' &&
                 (pline[epoint.pos + 2] | arguments.caseinsensitive) == 'n') {op = O_NOTIN;goto push2;}
             err_msg2(ERROR______EXPECTED, "an operator is", &epoint);
@@ -1916,10 +1950,8 @@ static bool get_exp2(int stop) {
             case 2: if ((pline[epoint.pos] | arguments.caseinsensitive) == 'i' &&
                         (pline[epoint.pos + 1] | arguments.caseinsensitive) == 'n') {op = O_IN;goto push2a;} break;
             }
-            if (ch < ' ' || ch > '~') {
-                if (err_msg_wrong_character(&epoint)) goto error;
-            }
-            err_msg2(ERROR______EXPECTED, "an operator is", &epoint);
+            if (ch < ' ' || ch > '~') err_msg_wrong_character(&epoint);
+            else err_msg2(ERROR______EXPECTED, "an operator is", &epoint);
             goto error;
         }
         while (opr.p != opr.data) {
@@ -1999,7 +2031,7 @@ MUST_CHECK Obj *calc2_lxor(oper_t op, bool i) {
     if (result->obj != BOOL_OBJ) return result;
     if (i == Bool(result)->value) {
         if (i) {
-            val_destroy(result); 
+            val_destroy(result);
             result = ref_false();
         }
         return result;
