@@ -1,5 +1,5 @@
 /*
-    $Id: foldobj.h 1925 2019-08-25 11:09:57Z soci $
+    $Id: foldobj.h 2651 2021-05-09 19:33:48Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,18 +22,22 @@
 
 extern struct Type *const FOLD_OBJ;
 
+struct Str;
+
 typedef struct Fold {
     Obj v;
-    int *dummy;
+    struct Str *repr;
 } Fold;
 
-extern Fold *fold_value;
+#define Fold(a) OBJ_CAST(Fold, a)
+
+extern Obj *const fold_value;
 
 extern void foldobj_init(void);
 extern void foldobj_destroy(void);
 
-static inline Fold *ref_fold(void) {
-    fold_value->v.refcount++; return fold_value;
+static inline Obj *ref_fold(void) {
+    fold_value->refcount++; return fold_value;
 }
 
 #endif
