@@ -1,5 +1,5 @@
 /*
-    $Id: functionobj.c 2896 2022-11-05 05:33:41Z soci $
+    $Id: functionobj.c 3096 2023-09-03 10:24:03Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -102,7 +102,7 @@ static MUST_CHECK Obj *gen_broadcast(oper_t op, apply_func_t f) {
     Funcargs *vals = Funcargs(op->v2);
     struct values_s *v = vals->val;
     argcount_t args = vals->len;
-    argcount_t j, k;
+    argcount_t j, k = 0;
     size_t ln = 1;
     List *vv;
     struct elements_s {
@@ -117,7 +117,7 @@ static MUST_CHECK Obj *gen_broadcast(oper_t op, apply_func_t f) {
             struct iter_s *iter;
             if (elements == NULL) {
                 if (args <= lenof(elements3)) {
-                    elements = elements3; 
+                    elements = elements3;
                 } else {
                     elements = allocate_array(struct elements_s, args);
                     if (elements == NULL) goto failed;
@@ -244,7 +244,6 @@ void random_reseed(Obj *o1, linepos_t epoint) {
         else if (v->obj == ERROR_OBJ) err_msg_output(Error(v));
     } else {
         Int *v1 = Int(v);
-        Error *err;
 
         state[0] = (((uint64_t)0x5229a30f) << 32) | (uint64_t)0x996ad7eb;
         state[1] = (((uint64_t)0xc03bbc75) << 32) | (uint64_t)0x3f671f6f;
@@ -256,10 +255,14 @@ void random_reseed(Obj *o1, linepos_t epoint) {
         case 1: state[0] ^= v1->data[0]; FALL_THROUGH; /* fall through */
         case 0: break;
         default:
-            err = new_error(v1->len < 0 ? ERROR______NOT_UVAL : ERROR_____CANT_UVAL, epoint);
-            err->u.intconv.bits = 128;
-            err->u.intconv.val = val_reference(o1);
-            err_msg_output_and_destroy(err);
+            if (v1->len < 0) {
+                err_msg2(ERROR______NOT_UVAL, o1, epoint);
+            } else {
+                Error *err = new_error(ERROR_____CANT_UVAL, epoint);
+                err->u.intconv.bits = 128;
+                err->u.intconv.val = val_reference(o1);
+                err_msg_output_and_destroy(err);
+            }
         }
     }
     val_destroy(v);
@@ -275,7 +278,7 @@ static MUST_CHECK Obj *function_random(oper_t op) {
 
     switch (vals->len) {
     default:
-        return new_float((double)(random64() & (((uint64_t)1 << 53) - 1)) * ldexp(1, -53));
+        return new_float((double)(int64_t)(random64() & (((uint64_t)1 << 53) - 1)) * ldexp(1, -53));
     case 1:
         err = v[0].val->obj->ival(v[0].val, &end, 8 * sizeof end, &v[0].epoint);
         break;
@@ -431,6 +434,7 @@ static MUST_CHECK Obj *function_binary(oper_t op) {
 
     err = Error(tostr2(&v[0], &filename));
     if (err != NULL) return Obj(err);
+    if (filename.len == 0) return Obj(new_error(ERROR__EMPTY_STRING, &v[0].epoint));
     cfile2 = file_open(&filename, current_file_list, FILE_OPEN_BINARY, &v[0].epoint);
 
     switch (vals->len) {
@@ -445,7 +449,7 @@ static MUST_CHECK Obj *function_binary(oper_t op) {
     default:
         break;
     }
-    
+
     if (cfile2 != NULL) {
         filesize_t offset, ln = cfile2->binary.len;
         Bytes *b;
@@ -617,7 +621,7 @@ static MUST_CHECK Obj *apply_func(oper_t op, apply_func_t f) {
             iter_destroy(&iter);
             v->len = i;
             return Obj(v);
-        } 
+        }
         v = List(val_reference(o2));
         len = v->len;
         vals = v->data;
@@ -678,10 +682,10 @@ static MUST_CHECK Obj *function_pow(oper_t op) {
     val = to_real(&v[1], &real2);
     if (val != NULL) return val;
     if (real2 < 0.0 && real == 0.0) {
-        return new_error_obj(ERROR_ZERO_NEGPOWER, op->v2, op->epoint3);
+        return new_error_obj(ERROR_ZERO_NEGPOWER, op->v2, op->epoint2);
     }
     if (real < 0.0 && floor(real2) != real2) {
-        return Obj(new_error(ERROR_NEGFRAC_POWER, op->epoint));
+        return Obj(new_error(ERROR_NEGFRAC_POWER, op->epoint2));
     }
     return float_from_double(pow(real, real2), op->epoint);
 }
@@ -740,7 +744,7 @@ static MUST_CHECK Obj *function_condition(oper_t op) {
         cond = Bool(val)->value;
         if (diagnostics.strict_bool) err_msg_bool(ERROR_____CANT_BOOL, v->val, &v->epoint);
         val_destroy(val);
-    } 
+    }
     return val_reference(v[cond ? 1 : 2].val);
 }
 
@@ -769,42 +773,42 @@ static MUST_CHECK Obj *calc2(oper_t op) {
                 switch (func) {
                 case F_HYPOT:
                     if (args != 2) {
-                        return new_error_argnum(args, 2, 2, op->epoint2);
+                        return new_error_argnum(args, 2, 2, op->epoint3);
                     }
                     return gen_broadcast(op, function_hypot);
                 case F_ATAN2:
                     if (args != 2) {
-                        return new_error_argnum(args, 2, 2, op->epoint2);
+                        return new_error_argnum(args, 2, 2, op->epoint3);
                     }
                     return gen_broadcast(op, function_atan2);
                 case F_POW:
                     if (args != 2) {
-                        return new_error_argnum(args, 2, 2, op->epoint2);
+                        return new_error_argnum(args, 2, 2, op->epoint3);
                     }
                     return gen_broadcast(op, function_pow);
                 case F_RANGE:
                     if (args < 1 || args > 3) {
-                        return new_error_argnum(args, 1, 3, op->epoint2);
+                        return new_error_argnum(args, 1, 3, op->epoint3);
                     }
                     return gen_broadcast(op, function_range);
                 case F_BINARY:
                     if (args < 1 || args > 3) {
-                        return new_error_argnum(args, 1, 3, op->epoint2);
+                        return new_error_argnum(args, 1, 3, op->epoint3);
                     }
                     return gen_broadcast(op, function_binary);
                 case F_FORMAT:
                     if (args < 1) {
-                        return new_error_argnum(args, 1, 0, op->epoint2);
+                        return new_error_argnum(args, 1, 0, op->epoint3);
                     }
                     return gen_broadcast(op, isnprintf);
                 case F_RANDOM:
                     if (args > 3) {
-                        return new_error_argnum(args, 0, 3, op->epoint2);
+                        return new_error_argnum(args, 0, 3, op->epoint3);
                     }
                     return gen_broadcast(op, function_random);
                 default:
                     if (args != 1) {
-                        return new_error_argnum(args, 1, 1, op->epoint2);
+                        return new_error_argnum(args, 1, 1, op->epoint3);
                     }
                     switch (func) {
                     case F_ANY:
@@ -818,12 +822,12 @@ static MUST_CHECK Obj *calc2(oper_t op) {
                             }
                             return ret;
                         }
-                    case F_LEN: 
+                    case F_LEN:
                         op->v2 = v->val;
                         return v->val->obj->len(op);
                     case F_SORT:
                         return function_sort(v->val, &v->epoint);
-                    default: 
+                    default:
                         op->v2 = v->val;
                         op->inplace = v->val->refcount == 1 ? v->val : NULL;
                         return apply_func(op, function_function);
