@@ -1,5 +1,5 @@
 /*
-    $Id: functionobj.h 2123 2019-12-21 06:42:26Z soci $
+    $Id: functionobj.h 2884 2022-10-31 13:29:58Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 #ifndef FUNCTIONOBJ_H
 #define FUNCTIONOBJ_H
 #include "obj.h"
-#include "str.h"
 
 extern struct Type *const FUNCTION_OBJ;
 
@@ -28,21 +27,30 @@ typedef enum Function_types {
     F_LOG10, F_EXP, F_SIN, F_COS, F_TAN, F_ACOS, F_ASIN, F_ATAN, F_RAD, F_DEG,
     F_COSH, F_SINH, F_TANH, F_HYPOT, F_ATAN2, F_POW, F_SIGN, F_ABS, F_ALL,
     F_ANY, F_SIZE, F_LEN, F_RANGE, F_REPR, F_FORMAT, F_RANDOM, F_SORT,
-    F_BINARY
+    F_BINARY, F_BYTE, F_CHAR, F_RTA, F_ADDR, F_SINT, F_WORD, F_LINT, F_LONG,
+    F_DINT, F_DWORD
 } Function_types;
 
 typedef struct Function {
     Obj v;
-    str_t name;
+    const char name[7];
+    const uint8_t name_len;
     int name_hash;
-    Function_types func;
+    const Function_types func;
 } Function;
 
+#define Function(a) OBJ_CAST(Function, a)
+
 struct oper_s;
 
-extern MUST_CHECK Obj *apply_convert(struct oper_s *);
+typedef MUST_CHECK Obj *(*apply_func_t)(struct oper_s *);
+
+extern MUST_CHECK Obj *apply_function(struct oper_s *, apply_func_t);
+extern MUST_CHECK Obj *apply_convert2(struct oper_s *);
+extern MUST_CHECK Obj *apply_condition(struct oper_s *);
 extern void functionobj_init(void);
 extern void functionobj_names(void);
+extern void functionobj_destroy(void);
 extern void random_reseed(Obj *, linepos_t);
 
 #endif
