@@ -1,5 +1,5 @@
 /*
-    $Id: instruction.c 2959 2023-01-08 10:32:44Z soci $
+    $Id: instruction.c 3086 2023-09-03 06:23:08Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -150,9 +150,9 @@ static void dump_instr(unsigned int cod, uint32_t adr, int ln, linepos_t epoint)
     listing_instr(cod, adr, ln);
 }
 
-typedef enum Adrgen { 
+typedef enum Adrgen {
     AG_ZP, AG_B0, AG_PB, AG_BYTE, AG_SBYTE, AG_CHAR, AG_DB3, AG_DB2, AG_WORD,
-    AG_SWORD, AG_SINT, AG_RELPB, AG_RELL, AG_IMP, AG_NONE 
+    AG_SWORD, AG_SINT, AG_RELPB, AG_RELL, AG_IMP, AG_NONE
 } Adrgen;
 
 static Adrgen adrmatch(const uint8_t *cnmemonic, int prm, atype_t am, unsigned int w, Adr_types *opr) {
@@ -313,11 +313,10 @@ static Adrgen adrmatch(const uint8_t *cnmemonic, int prm, atype_t am, unsigned i
             break;
         }
         return AG_NONE;
-    case (A_IMMEDIATE_SIGNED << 12) | (A_SR << 8) | (A_I << 4) | A_YR:/* lda (#+$ff,s),y */
     case (A_IMMEDIATE << 12) | (A_SR << 8) | (A_I << 4) | A_YR:/* lda (#$ff,s),y */
     case (A_SR << 8) | (A_I << 4) | A_YR:
         if (cnmemonic[ADR_ZP_S_I_Y] != ____) {
-            adrgen = (opcode == c65ce02.opcode || opcode == c4510.opcode) ? AG_CHAR : AG_BYTE; *opr = ADR_ZP_S_I_Y; /* lda ($ff,s),y */
+            adrgen = AG_BYTE; *opr = ADR_ZP_S_I_Y; /* lda ($ff,s),y */
             break;
         }
         return AG_NONE;
@@ -455,7 +454,7 @@ MUST_CHECK Error *instruction(int prm, unsigned int w, Funcargs *vals, linepos_t
                 case ADR_REL:
                     ln = 1; longbranch = 0;
                     goto justrel2;
-                case ADR_IMMEDIATE: 
+                case ADR_IMMEDIATE:
                     if (cnmemonic[ADR_REL] != ____) {
                         ln = 1; longbranch = 0; opr = ADR_REL;
                         goto immediaterel;
@@ -465,7 +464,7 @@ MUST_CHECK Error *instruction(int prm, unsigned int w, Funcargs *vals, linepos_t
                         ln = 2; longbranch = 0; opr = ADR_REL_L;
                         goto immediaterel;
                     }
-                    if (pline[epoint2->pos] == '#') epoint2->pos++; 
+                    if (pline[epoint2->pos] == '#') epoint2->pos++;
                     break;
                 case ADR_ZP_I_Y:
                 case ADR_ZP_S_I_Y:
@@ -474,12 +473,12 @@ MUST_CHECK Error *instruction(int prm, unsigned int w, Funcargs *vals, linepos_t
                 case ADR_ZP_X_I:
                 case ADR_ADDR_I:
                 case ADR_ZP_I:
-                    if (pline[epoint2->pos] == '(') epoint2->pos++; 
+                    if (pline[epoint2->pos] == '(') epoint2->pos++;
                     break;
                 case ADR_ZP_LI_Y:
                 case ADR_ADDR_LI:
                 case ADR_ZP_LI:
-                    if (pline[epoint2->pos] == '[') epoint2->pos++; 
+                    if (pline[epoint2->pos] == '[') epoint2->pos++;
                     break;
                 default:
                     break;
@@ -839,6 +838,16 @@ MUST_CHECK Error *instruction(int prm, unsigned int w, Funcargs *vals, linepos_t
         if (vals->val[0].val->obj->iterable || vals->val[1].val->obj->iterable || vals->val[2].val->obj->iterable) {
             goto broadcast;
         }
+        if (vals->val[1].val->obj == REGISTER_OBJ && vals->val[2].val->obj == REGISTER_OBJ) {
+            atype_t am2 = Register(vals->val[2].val)->len != 1 ? A_NONE : register_to_indexing(Register(vals->val[2].val)->data[0]);
+            am = Register(vals->val[1].val)->len != 1 ? A_NONE : register_to_indexing(Register(vals->val[1].val)->data[0]);
+            if (am != A_NONE && am2 != A_NONE) {
+                val = vals->val[0].val;
+                am |= val->obj->address(val) << 4;
+                am = (am << 4) | am2;
+                goto retry1;
+            }
+        }
         if (cnmemonic[ADR_BIT_ZP_REL] != ____) {
             if (w != 3 && w != 1) return err_addressize((w != 0) ? ERROR__NO_LONG_ADDR : ERROR__NO_BYTE_ADDR, epoint2, prm);
             if (touval(vals->val[0].val, &uval, 3, epoint2)) {}
@@ -1069,13 +1078,13 @@ MUST_CHECK Error *instruction(int prm, unsigned int w, Funcargs *vals, linepos_t
                     else if ((uval & all_mem) != uval) err_msg_addr_wrap(epoint2);
                     break;
                 case 2:
-                    if (adrgen == AG_DB3 && cnmemonic[opr - 2] != ____) { 
+                    if (adrgen == AG_DB3 && cnmemonic[opr - 2] != ____) {
                         adr = uval & all_mem;
                         if ((uval & all_mem) != uval) err_msg_addr_wrap(epoint2);
                         break;
                     }
                     FALL_THROUGH; /* fall through */
-                default: 
+                default:
                     return err_addressize(ERROR__NO_LONG_ADDR, epoint2, prm);
                 }
             }
