@@ -1,5 +1,5 @@
 /*
-    $Id: listing.c 2924 2022-12-22 08:52:34Z soci $
+    $Id: listing.c 3112 2023-09-06 06:34:22Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -49,14 +49,11 @@ typedef struct Listing {
     char *s;
     char buf[LINE_WIDTH + ADDR_WIDTH + LADDR_WIDTH + 3*16];
     char hex[16];
-    const uint16_t *disasm;
-    const uint32_t *mnemonic;
     struct {
         unsigned int addr, laddr, hex, monitor, source;
     } columns;
     FILE *flist;
     uint16_t lastfile;
-    const char *filename;
     unsigned int tab_size;
     bool linenum, verbose, monitor, pccolumn, source;
 } Listing;
@@ -149,7 +146,7 @@ void listing_open(const struct list_output_s *output, int argc, char *argv[]) {
 
     flist = dash_name(output->name) ? stdout : fopen_utf8(output->name, output->append ? "at" : "wt");
     if (flist == NULL) {
-        err_msg_file2(ERROR_CANT_WRTE_LST, output->name);
+        err_msg_file2(ERROR_CANT_WRTE_LST, output->name, &output->name_pos);
         listing = NULL;
         return;
     }
@@ -159,7 +156,6 @@ void listing_open(const struct list_output_s *output, int argc, char *argv[]) {
     ls = &listing2;
 
     memcpy(ls->hex, "0123456789abcdef", 16);
-    ls->filename = output->name;
     ls->flist = flist;
     ls->linenum = arguments.list.linenum;
     ls->pccolumn = listing_pccolumn;
@@ -182,7 +178,8 @@ void listing_open(const struct list_output_s *output, int argc, char *argv[]) {
         argv_print(argv[i], flist);
     }
     fputs("\n; ", flist);
-    time(&t); fputs(ctime(&t), flist);
+    if (get_latest_file_time((void *)&t)) time(&t);
+    fputs(ctime(&t), flist);
     newline(ls);
     if (ls->linenum) {
         out_txt(ls, ";Line");
@@ -208,7 +205,7 @@ void listing_open(const struct list_output_s *output, int argc, char *argv[]) {
     listing = ls;
 }
 
-void listing_close(void) {
+void listing_close(const struct list_output_s *output) {
     Listing *const ls = listing;
     int err;
     if (ls == NULL) return;
@@ -216,7 +213,7 @@ void listing_close(void) {
     fputs("\n;******  End of listing\n", ls->flist);
     err = ferror(ls->flist);
     err |= (ls->flist != stdout) ? fclose(ls->flist) : fflush(ls->flist);
-    if (err != 0 && errno != 0) err_msg_file2(ERROR_CANT_WRTE_LST, ls->filename);
+    if (err != 0 && errno != 0) err_msg_file2(ERROR_CANT_WRTE_LST, output->name, &output->name_pos);
     listing = NULL;
 }
 
@@ -255,7 +252,7 @@ static void printfile(Listing *ls) {
 
 static void printline(Listing *ls) {
     uint16_t curfile = current_file_list->file->uid;
-    if (curfile < 2) return;
+    if (current_file_list->file->notfile) return;
     printdec(ls, lpoint.line);
     if (ls->lastfile == curfile) return;
     ls->lastfile = curfile;
@@ -320,14 +317,14 @@ static void printmon(Listing *ls, unsigned int cod, int ln, uint32_t adr) {
     uint32_t mnem;
 
     padding2(ls, ls->columns.monitor);
-    mnem = ls->mnemonic[ls->disasm[cod] & 0xff];
+    mnem = current_cpu->mnemonic[current_cpu->disasm[cod] & 0xff];
     ls->s[0] = (char)(mnem >> 16);
     ls->s[1] = (char)(mnem >> 8);
     ls->s[2] = (char)(mnem);
     ls->s[3] = ' ';
     ls->s += 4;
 
-    type = (Adr_types)(ls->disasm[cod] >> 8);
+    type = (Adr_types)(current_cpu->disasm[cod] >> 8);
     mode = addr_modes[type];
     if (*mode != ' ') *ls->s++ = *mode;
     mode++;
@@ -390,6 +387,7 @@ FAST_CALL void listing_equal2(Obj *val, linecpos_t pos) {
     if (ls->linenum) {
         printline(ls);
         padding2(ls, ls->columns.addr);
+        flushbuf(ls);
     }
     putc('=', ls->flist);
     ls->c += val_print(val, ls->flist, ls->verbose ? SIZE_MAX : ls->columns.source - 2) + 1;
@@ -482,13 +480,6 @@ FAST_CALL void listing_line_cut2(linecpos_t pos) {
     }
 }
 
-FAST_CALL void listing_set_cpumode(const struct cpu_s *cpumode) {
-    Listing *const ls = listing;
-    if (ls == NULL) return;
-    ls->disasm = cpumode->disasm;
-    ls->mnemonic = cpumode->mnemonic;
-}
-
 void listing_instr(unsigned int cod, uint32_t adr, int ln) {
     Listing *const ls = listing;
     address_t addr, addr2;
