@@ -1,5 +1,5 @@
 /*
-    $Id: longjump.c 1961 2019-09-04 03:35:09Z soci $
+    $Id: longjump.c 2708 2021-09-18 18:12:25Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,15 +17,13 @@
 
 */
 #include "longjump.h"
-#include "section.h"
 #include "error.h"
 
 static FAST_CALL int longjump_compare(const struct avltree_node *aa, const struct avltree_node *bb)
 {
-    const struct longjump_s *a = cavltree_container_of(aa, struct longjump_s, node);
-    const struct longjump_s *b = cavltree_container_of(bb, struct longjump_s, node);
-    if (a->address != b->address) return a->address > b->address ? 1 : -1;
-    return 0;
+    address_t a = cavltree_container_of(aa, struct longjump_s, node)->address;
+    address_t b = cavltree_container_of(bb, struct longjump_s, node)->address;
+    return (a > b) - (a < b);
 }
 
 static void longjump_free(struct avltree_node *aa)
@@ -34,29 +32,26 @@ static void longjump_free(struct avltree_node *aa)
     free(a);
 }
 
-static struct longjump_s *lastlj = NULL;
-struct longjump_s *new_longjump(address_t address, bool *exists) {
+static struct longjump_s *lastlj;
+struct longjump_s *new_longjump(struct avltree *t, address_t address) {
     struct avltree_node *b;
     struct longjump_s *tmp;
 
-    if (lastlj == NULL) {
-        lastlj = (struct longjump_s *)mallocx(sizeof *lastlj);
-    }
+    if (lastlj == NULL) new_instance(&lastlj);
     lastlj->address = address;
-    b = avltree_insert(&lastlj->node, &current_section->longjump, longjump_compare);
+    b = avltree_insert(&lastlj->node, t, longjump_compare);
     if (b == NULL) { /* new longjump */
         lastlj->defpass = 0;
-        *exists = false;
         tmp = lastlj;
         lastlj = NULL;
         return tmp;
     }
-    *exists = true;
     return avltree_container_of(b, struct longjump_s, node);            /* already exists */
 }
 
 void destroy_longjump(void) {
     free(lastlj);
+    lastlj = NULL;
 }
 
 void longjump_destroy(struct avltree *t) {
