@@ -1,5 +1,5 @@
 /*
-    $Id: macro.c 2896 2022-11-05 05:33:41Z soci $
+    $Id: macro.c 3086 2023-09-03 06:23:08Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -263,7 +263,7 @@ bool mtranslate(void) {
                 param.data = NULL;
                 param.len = 0;
             }
-        } 
+        }
     tasmc:
         if (p + param.len < p) err_msg_out_of_memory(); /* overflow */
         if (p + param.len > mline->len) {
@@ -281,15 +281,15 @@ bool mtranslate(void) {
         mline->rpositions[n].param = j;
         mline->rpositions[n++].len = (linecpos_t)param.len;
         switch (param.len) {
-        case 0: 
+        case 0:
             if (param.data == NULL) {
                 lpoint.pos = (linecpos_t)(last2 - pline) + p - last;
                 err_msg_missing_argument(&lpoint, j);
                 fault = true;
             }
             break;
-        case 1: 
-            mline->data[p++] = *param.data; 
+        case 1:
+            mline->data[p++] = *param.data;
             break;
         default:
             memcpy(mline->data + p, param.data, param.len);
@@ -600,7 +600,7 @@ Obj *mfunc_recurse(Mfunc *mfunc, Namespace *context, uint8_t strength, linepos_t
 
 bool get_func_params(Mfunc *v, bool single) {
     struct mfunc_param_s *params;
-    argcount_t len = v->argc, i = 0, j;
+    argcount_t len = v->argc, i = 0;
     str_t label;
     bool stard = false, ret = false;
 
@@ -642,14 +642,6 @@ bool get_func_params(Mfunc *v, bool single) {
                 str_cfcpy(&param->cfname, &label);
                 if (param->cfname.data != label.data) str_cfcpy(&param->cfname, NULL);
                 else param->cfname = param->name;
-                for (j = 0; j < i; j++) {
-                    if (params[j].name.data != NULL) {
-                        if (str_cmp(&params[j].cfname, &param->cfname) == 0) break;
-                    }
-                }
-                if (j != i) {
-                    err_msg_double_definedo(v->file_list, &params[j].epoint, &label, &param->epoint);
-                }
             } else {
                 err_msg2(ERROR_GENERL_SYNTAX, NULL, &param->epoint);
                 ret = true;
