@@ -1,5 +1,5 @@
 /*
-    $Id: opcodes.c 2156 2020-03-08 12:44:05Z soci $
+    $Id: opcodes.c 2709 2021-09-18 18:40:01Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ static const uint16_t disasm_w65816[] = {
     0x0151, 0x0a21, 0x0274, 0x0b21, 0x173b, 0x0521, 0x0539, 0x1221,
     0x0141, 0x0221, 0x0039, 0x0144, 0x0433, 0x0421, 0x0439, 0x0321,
     0x160f, 0x1021, 0x1421, 0x0c21, 0x173a, 0x0821, 0x0839, 0x0f21,
-    0x0113, 0x0d21, 0x0147, 0x0165, 0x0333, 0x0721, 0x0739, 0x0621,
+    0x0113, 0x0d21, 0x0147, 0x0165, 0x0332, 0x0721, 0x0739, 0x0621,
     0x0153, 0x0a00, 0x1540, 0x0b00, 0x055f, 0x0500, 0x0550, 0x1200,
     0x0148, 0x0200, 0x0050, 0x0152, 0x1333, 0x0400, 0x0450, 0x0300,
     0x1610, 0x1000, 0x1400, 0x0c00, 0x085f, 0x0800, 0x0850, 0x0f00,
@@ -849,7 +849,7 @@ const struct cpu_s c4510 = {
 
 const char *reg_names = "axysdrizbkp";
 
-const char * const addr_modes[ADR_LEN+1] = {
+const char *const addr_modes[ADR_LEN+1] = {
     "a",      /* ADR_REG */
     " ",      /* ADR_IMPLIED */
     "#",      /* ADR_IMMEDIATE */
@@ -916,18 +916,18 @@ const uint8_t opcode_table[][ADR_LEN] = {
     {  0, ____, 0x69, 0x6F, 0x6D, 0x65, 0x7F, 0x7D, 0x75, ____, 0x61, 0x63, 0x73, 0x79, ____, 0x77, 0x71, ____, 0x67, ____, 0x72, ____, ____, ____, ____, ____, ____, ____ }, /* ADC */
     {  0, ____, 0x29, 0x2F, 0x2D, 0x25, 0x3F, 0x3D, 0x35, ____, 0x21, 0x23, 0x33, 0x39, ____, 0x37, 0x31, ____, 0x27, ____, 0x32, ____, ____, ____, ____, ____, ____, ____ }, /* AND */
     {  1, 0x0A, ____, ____, 0x0E, 0x06, ____, 0x1E, 0x16, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* ASL, SHL */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x90, ____, ____, ____, ____, ____ }, /* BCC, BLT */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xB0, ____, ____, ____, ____, ____ }, /* BCS, BGE */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xF0, ____, ____, ____, ____, ____ }, /* BEQ */
+    {  0, ____, 0x90, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x90, ____, ____, ____, ____, ____ }, /* BCC, BLT */
+    {  0, ____, 0xB0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xB0, ____, ____, ____, ____, ____ }, /* BCS, BGE */
+    {  0, ____, 0xF0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xF0, ____, ____, ____, ____, ____ }, /* BEQ */
     {  0, ____, 0x89, ____, 0x2C, 0x24, ____, 0x3C, 0x34, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* BIT */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x30, ____, ____, ____, ____, ____ }, /* BMI */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xD0, ____, ____, ____, ____, ____ }, /* BNE */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x10, ____, ____, ____, ____, ____ }, /* BPL */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x80, ____, ____, ____, ____, ____ }, /* BRA */
+    {  0, ____, 0x30, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x30, ____, ____, ____, ____, ____ }, /* BMI */
+    {  0, ____, 0xD0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xD0, ____, ____, ____, ____, ____ }, /* BNE */
+    {  0, ____, 0x10, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x10, ____, ____, ____, ____, ____ }, /* BPL */
+    {  0, ____, 0x80, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x80, ____, ____, ____, ____, ____ }, /* BRA */
     {  0, 0x00, 0x00, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* BRK */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x82, ____, ____, ____, ____, ____, ____ }, /* BRL */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x50, ____, ____, ____, ____, ____ }, /* BVC */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x70, ____, ____, ____, ____, ____ }, /* BVS */
+    {  0, ____, 0x82, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x82, ____, ____, ____, ____, ____, ____ }, /* BRL */
+    {  0, ____, 0x50, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x50, ____, ____, ____, ____, ____ }, /* BVC */
+    {  0, ____, 0x70, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x70, ____, ____, ____, ____, ____ }, /* BVS */
     {  0, 0x18, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* CLC */
     {  0, 0xD8, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* CLD */
     {  0, 0x58, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* CLI */
@@ -958,7 +958,7 @@ const uint8_t opcode_table[][ADR_LEN] = {
     {  0, 0xE8, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* INX */
     {  0, 0xC8, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* INY */
     {  0, ____, ____, 0x5C, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xDC, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* JML */
-    {  0, ____, ____, 0x5C, 0x4C, ____, ____, ____, ____, 0x7C, ____, ____, ____, ____, ____, ____, ____, 0xDC, ____, 0x6C, ____, ____, ____, ____, ____, ____, ____, ____ }, /* JMP */
+    {  0, ____, ____, ____, 0x4C, ____, ____, ____, ____, 0x7C, ____, ____, ____, ____, ____, ____, ____, 0xDC, ____, 0x6C, ____, ____, ____, ____, ____, ____, ____, ____ }, /* JMP */
     {  0, ____, ____, 0x22, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* JSL */
     {  0, ____, ____, ____, 0x20, ____, ____, ____, ____, 0xFC, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* JSR */
     {  4, ____, 0xA9, 0xAF, 0xAD, 0xA5, 0xBF, 0xBD, 0xB5, ____, 0xA1, 0xA3, 0xB3, 0xB9, ____, 0xB7, 0xB1, ____, 0xA7, ____, 0xB2, ____, ____, ____, ____, ____, ____, ____ }, /* LDA */
@@ -971,7 +971,7 @@ const uint8_t opcode_table[][ADR_LEN] = {
     {  0, ____, 0x09, 0x0F, 0x0D, 0x05, 0x1F, 0x1D, 0x15, ____, 0x01, 0x03, 0x13, 0x19, ____, 0x17, 0x11, ____, 0x07, ____, 0x12, ____, ____, ____, ____, ____, ____, ____ }, /* ORA */
     {  0, ____, 0xF4, ____, 0xF4, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* PEA */
     {  0, ____, ____, ____, ____, 0xD4, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xD4, ____, ____, ____, ____, ____, ____, ____ }, /* PEI */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x62, ____, ____, ____, ____, ____, ____ }, /* PER */
+    {  0, ____, 0x62, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x62, ____, ____, ____, ____, ____, ____ }, /* PER */
     {  0, 0x48, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* PHA */
     {  0, 0x8B, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* PHB, TXR */
     {  0, 0x0B, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* PHD, RHI, TSY */
@@ -1061,7 +1061,7 @@ const uint8_t opcode_table[][ADR_LEN] = {
     {  0, ____, ____, ____, ____, ____, ____, 0x9C, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* SHY */
     {  0, ____, ____, ____, 0x0F, 0x07, ____, 0x1F, 0x17, ____, 0x03, ____, ____, 0x1B, ____, ____, 0x13, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* SLO */
     {  0, ____, ____, ____, 0x4F, 0x47, ____, 0x5F, 0x57, ____, 0x43, ____, ____, 0x5B, ____, ____, 0x53, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* SRE */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x12, ____, ____, ____, ____, ____ }, /* BRA */
+    {  0, ____, 0x12, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x12, ____, ____, ____, ____, ____ }, /* BRA */
     {  0, ____, ____, ____, 0x4C, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x12, ____, ____, ____, ____, ____ }, /* GRA */
     {  0, ____, 0x32, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* SAC */
     {  0, ____, 0x69, ____, 0x6D, 0x65, ____, 0x7D, 0x75, ____, 0x61, 0x63, 0x73, 0x79, ____, ____, 0x71, ____, ____, ____, 0x72, ____, ____, ____, 0x67, 0x77, ____, ____ }, /* ADC */
@@ -1096,16 +1096,16 @@ const uint8_t opcode_table[][ADR_LEN] = {
     {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x87, ____ }, /* SMB */
     { 19, 0x43, ____, ____, ____, 0x44, ____, ____, 0x54, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* ASR */
     {  0, ____, ____, ____, 0xCB, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* ASW */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x93, 0x90, ____, ____, ____, ____, ____ }, /* BCC, BLT */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xB3, 0xB0, ____, ____, ____, ____, ____ }, /* BCS, BGE */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xF3, 0xF0, ____, ____, ____, ____, ____ }, /* BEQ */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x33, 0x30, ____, ____, ____, ____, ____ }, /* BMI */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xD3, 0xD0, ____, ____, ____, ____, ____ }, /* BNE */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x13, 0x10, ____, ____, ____, ____, ____ }, /* BPL */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x83, 0x80, ____, ____, ____, ____, ____ }, /* BRA */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x63, ____, ____, ____, ____, ____, ____ }, /* BSR */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x53, 0x50, ____, ____, ____, ____, ____ }, /* BVC */
-    {  0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x73, 0x70, ____, ____, ____, ____, ____ }, /* BVS */
+    {  0, ____, 0x90, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x93, 0x90, ____, ____, ____, ____, ____ }, /* BCC, BLT */
+    {  0, ____, 0xB0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xB3, 0xB0, ____, ____, ____, ____, ____ }, /* BCS, BGE */
+    {  0, ____, 0xF0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xF3, 0xF0, ____, ____, ____, ____, ____ }, /* BEQ */
+    {  0, ____, 0x30, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x33, 0x30, ____, ____, ____, ____, ____ }, /* BMI */
+    {  0, ____, 0xD0, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0xD3, 0xD0, ____, ____, ____, ____, ____ }, /* BNE */
+    {  0, ____, 0x10, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x13, 0x10, ____, ____, ____, ____, ____ }, /* BPL */
+    {  0, ____, 0x80, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x83, 0x80, ____, ____, ____, ____, ____ }, /* BRA */
+    {  0, ____, 0x63, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x63, ____, ____, ____, ____, ____, ____ }, /* BSR */
+    {  0, ____, 0x50, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x53, 0x50, ____, ____, ____, ____, ____ }, /* BVC */
+    {  0, ____, 0x70, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, 0x73, 0x70, ____, ____, ____, ____, ____ }, /* BVS */
     {  0, ____, 0xC2, ____, 0xDC, 0xD4, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* CPZ */
     { 20, ____, ____, ____, 0xCE, 0xC6, ____, 0xDE, 0xD6, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* DEC */
     {  0, ____, ____, ____, ____, 0xC3, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____, ____ }, /* DEW */
