@@ -1,5 +1,5 @@
 /*
-    $Id: typeobj.c 2118 2019-12-21 05:43:46Z soci $
+    $Id: typeobj.c 2896 2022-11-05 05:33:41Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,26 +20,19 @@
 #include <string.h>
 #include "variables.h"
 #include "eval.h"
-#include "error.h"
 
 #include "strobj.h"
-#include "operobj.h"
-#include "intobj.h"
-#include "boolobj.h"
-#include "listobj.h"
-#include "noneobj.h"
-#include "errorobj.h"
 #include "functionobj.h"
 
 static Type obj;
 
 Type *const TYPE_OBJ = &obj;
 
-static struct Slot *values_free[32];
+static struct Slot *values_free[MAXIMUM_TYPE_LENGTH];
 
 #define ALIGN sizeof(int *)
 
-void new_type(Type *t, Type_types type, const char *name, size_t length) {
+Type *new_type(Type *t, Type_types type, const char *name, size_t length) {
     t->v.obj = TYPE_OBJ;
     t->v.refcount = 1;
     t->type = type;
@@ -47,30 +40,33 @@ void new_type(Type *t, Type_types type, const char *name, size_t length) {
     t->slot = &values_free[t->length];
     t->name = name;
     obj_init(t);
+    return t;
 }
 
-static MUST_CHECK Obj *create(Obj *v1, linepos_t UNUSED(epoint)) {
+static MUST_CHECK Obj *type_from_obj(Obj *v1, linepos_t UNUSED(epoint)) {
     switch (v1->obj->type) {
     case T_NONE:
-    case T_ERROR:
-    case T_TYPE: return val_reference(v1);
+    case T_ERROR: return val_reference(v1);
     default: break;
     }
-    return val_reference((Obj *)&v1->obj->v);
+    return val_reference((Obj *)Obj(v1->obj));
+}
+
+static MUST_CHECK Obj *convert(oper_t op) {
+    return type_from_obj(op->v2, op->epoint2);
 }
 
 static FAST_CALL bool same(const Obj *o1, const Obj *o2) {
     return o1 == o2;
 }
 
-static MUST_CHECK Error *hash(Obj *o1, int *hs, linepos_t UNUSED(epoint)) {
-    Type *v1 = (Type *)o1;
-    *hs = v1->type;
+static MUST_CHECK Obj *hash(Obj *o1, int *hs, linepos_t UNUSED(epoint)) {
+    *hs = (int)Type(o1)->type;
     return NULL;
 }
 
 static MUST_CHECK Obj *repr(Obj *o1, linepos_t epoint, size_t maxsize) {
-    Type *v1 = (Type *)o1;
+    Type *v1 = Type(o1);
     Str *v;
     uint8_t *s;
     const char *name;
@@ -89,57 +85,42 @@ static MUST_CHECK Obj *repr(Obj *o1, linepos_t epoint, size_t maxsize) {
     memcpy(s, name, ln);
     s[ln] = '\'';
     s[ln + 1] = '>';
-    return &v->v;
+    return Obj(v);
 }
 
-static inline int icmp(const Type *vv1, const Type *vv2) {
-    Type_types v1 = vv1->type;
-    Type_types v2 = vv2->type;
-    if (v1 < v2) return -1;
-    return (v1 > v2) ? 1 : 0;
+static inline int icmp(oper_t op) {
+    Type_types v1 = Type(op->v1)->type;
+    Type_types v2 = Type(op->v2)->type;
+    return (v1 < v2) ? -1 : (v1 > v2) ? 1 : 0;
 }
 
 static MUST_CHECK Obj *calc2(oper_t op) {
-    Type *v1 = (Type *)op->v1;
     Obj *o2 = op->v2;
 
     switch (o2->obj->type) {
     case T_TYPE:
-        {
-            Type *v2 = (Type *)o2;
-            int val = icmp(v1, v2);
-            switch (op->op->op) {
-            case O_CMP:
-                if (val < 0) return (Obj *)ref_int(minus1_value);
-                return (Obj *)ref_int(int_value[(val > 0) ? 1 : 0]);
-            case O_EQ: return truth_reference(val == 0);
-            case O_NE: return truth_reference(val != 0);
-            case O_MIN:
-            case O_LT: return truth_reference(val < 0);
-            case O_LE: return truth_reference(val <= 0);
-            case O_MAX:
-            case O_GT: return truth_reference(val > 0);
-            case O_GE: return truth_reference(val >= 0);
-            default: break;
-            }
-        }
-        break;
+        return obj_oper_compare(op, icmp(op));
     case T_FUNCARGS:
-        if (op->op == &o_FUNC) {
-            Funcargs *v2 = (Funcargs *)o2;
-            size_t args = v2->len;
+        if (op->op == O_FUNC) {
+            const Type *v1 = Type(op->v1);
+            Funcargs *v2 = Funcargs(o2);
+            argcount_t args = v2->len;
             if (args != 1) {
-                return (Obj *)new_error_argnum(args, 1, 1, op->epoint2);
+                return apply_convert2(op);
             }
-            if (v1 == LIST_OBJ || v1 == TUPLE_OBJ || v1 == TYPE_OBJ) return v1->create(v2->val[0].val, op->epoint);
-            return apply_convert(op);
+            op->v2 = v2->val->val;
+            op->inplace = op->v2->refcount == 1 ? op->v2 : NULL;
+            if (v1->iterable || v1 == TYPE_OBJ) {
+                return v1->convert(op);
+            }
+            return apply_function(op, v1->convert);
         }
         break;
     case T_NONE:
     case T_ERROR:
         return val_reference(o2);
     default:
-        if (o2->obj->iterable && op->op != &o_MEMBER && op->op != &o_X) {
+        if (o2->obj->iterable && op->op != O_MEMBER && op->op != O_X) {
             return o2->obj->rcalc2(op);
         }
         break;
@@ -147,16 +128,19 @@ static MUST_CHECK Obj *calc2(oper_t op) {
     return obj_oper_error(op);
 }
 
+void init_type(void) {
+    memset(values_free, 0, sizeof values_free);
+}
 
 void typeobj_init(void) {
-    new_type(&obj, T_TYPE, "type", sizeof(Type));
-    obj.create = create;
-    obj.same = same;
-    obj.hash = hash;
-    obj.repr = repr;
-    obj.calc2 = calc2;
+    Type *type = new_type(&obj, T_TYPE, "type", sizeof(Type));
+    type->convert = convert;
+    type->same = same;
+    type->hash = hash;
+    type->repr = repr;
+    type->calc2 = calc2;
 }
 
 void typeobj_names(void) {
-    new_builtin("type", val_reference(&TYPE_OBJ->v));
+    new_builtin("type", val_reference(Obj(TYPE_OBJ)));
 }
