@@ -1,5 +1,5 @@
 /*
-    $Id: unicode.c 2967 2023-01-08 22:50:27Z soci $
+    $Id: unicode.c 3086 2023-09-03 06:23:08Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -829,7 +829,7 @@ uint8_t *char_to_utf8(const char *s) {
         return (uint8_t *)s;
     }
     while (s[n] != '\0') n++;
-        
+
     data = add_overflow(n, 64, &len) ? NULL : allocate_array(uint8_t, len);
     if (data == NULL) return NULL;
 
@@ -954,6 +954,7 @@ const char *unicode_character_name(unichar_t ch) {
     case 0x20: txt = " space"; break;
     case 0x22: txt = " quotation mark"; break;
     case 0x27: txt = " apostrophe"; break;
+    case 0x2a: txt = " asterix"; break;
     case 0x2d: txt = " hyphen-minus"; break;
     case 0xa0: txt = " no-break space"; break;
     case 0x2010: txt = " hyphen"; break;
@@ -970,6 +971,7 @@ const char *unicode_character_name(unichar_t ch) {
     case 0x201E: txt = " double low-9 quotation mark"; break;
     case 0x201F: txt = " double high-reversed-9 quotation mark"; break;
     case 0x2212: txt = " minus sign"; break;
+    case 0x2217: txt = " asterix operator"; break;
     case 0xFEFF: txt = " zero width no-break space"; break;
     default: txt = NULL;
     }
