@@ -1,5 +1,5 @@
 /*
-    $Id: values.c 2177 2020-03-25 13:51:29Z soci $
+    $Id: values.c 2898 2022-11-05 08:08:41Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ typedef struct Slotcoll {
     struct Slotcoll *next;
 } Slotcoll;
 
-static Slotcoll *slotcoll[32];
+static Slotcoll *slotcoll[MAXIMUM_TYPE_LENGTH];
 
 static void value_free(Obj *val) {
     Slot *slot = (Slot *)val, **c = val->obj->slot;
@@ -48,16 +48,18 @@ static void value_free(Obj *val) {
 
 static FAST_CALL NO_INLINE Obj *value_alloc(const Type *obj) {
     size_t p = obj->length;
-    size_t i, size = p * ALIGN;
+    size_t size = p * ALIGN;
     Slot *slot, *slot2;
     Slotcoll **s = &slotcoll[p];
-    Slotcoll *n = (Slotcoll *)mallocx(size * SLOTS + sizeof *n);
+    size_t i = size * SLOTS + sizeof(Slotcoll);
+    Slotcoll *n = (Slotcoll *)allocate_array(uint8_t, i);
+    if (n == NULL) err_msg_out_of_memory();
     n->next = *s; *s = n;
     slot2 = slot = (Slot *)(n + 1);
     for (i = 0; i < (SLOTS - 1); i++, slot2 = slot2->next) {
         slot2->v.obj = NULL;
         slot2->v.refcount = 1;
-        slot2->next = (Slot *)(((const char *)slot2) + size);
+        slot2->next = (Slot *)(((char *)slot2) + size);
     }
     slot2->v.obj = NULL;
     slot2->v.refcount = 1;
@@ -65,7 +67,7 @@ static FAST_CALL NO_INLINE Obj *value_alloc(const Type *obj) {
 
     slot->v.obj = obj;
     *obj->slot = slot->next;
-    return &slot->v;
+    return Obj(slot);
 }
 
 FAST_CALL MALLOC Obj *val_alloc(const Type *obj) {
@@ -73,7 +75,7 @@ FAST_CALL MALLOC Obj *val_alloc(const Type *obj) {
     if (slot == NULL) return value_alloc(obj);
     slot->v.obj = obj;
     *obj->slot = slot->next;
-    return &slot->v;
+    return Obj(slot);
 }
 
 void garbage_collect(void) {
@@ -84,7 +86,7 @@ void garbage_collect(void) {
         size_t size = j * ALIGN;
         for (vals = slotcoll[j]; vals != NULL; vals = vals->next) {
             Obj *val = (Obj *)(vals + 1);
-            for (i = 0; i < SLOTS; i++, val = (Obj *)(((const char *)val) + size)) {
+            for (i = 0; i < SLOTS; i++, val = (Obj *)(((char *)val) + size)) {
                 if (val->obj != NULL && val->obj->garbage != NULL) {
                     val->obj->garbage(val, -1);
                     val->refcount |= SIZE_MSB;
@@ -97,7 +99,7 @@ void garbage_collect(void) {
         size_t size = j * ALIGN;
         for (vals = slotcoll[j]; vals != NULL; vals = vals->next) {
             Obj *val = (Obj *)(vals + 1);
-            for (i = 0; i < SLOTS; i++, val = (Obj *)(((const char *)val) + size)) {
+            for (i = 0; i < SLOTS; i++, val = (Obj *)(((char *)val) + size)) {
                 if (val->obj != NULL && val->obj->garbage != NULL) {
                     if (val->refcount > SIZE_MSB) {
                         val->refcount -= SIZE_MSB;
@@ -112,7 +114,7 @@ void garbage_collect(void) {
         size_t size = j * ALIGN;
         for (vals = slotcoll[j]; vals != NULL; vals = vals->next) {
             Obj *val = (Obj *)(vals + 1);
-            for (i = 0; i < SLOTS; i++, val = (Obj *)(((const char *)val) + size)) {
+            for (i = 0; i < SLOTS; i++, val = (Obj *)(((char *)val) + size)) {
                 if ((val->refcount & ~SIZE_MSB) == 0) {
                     val->refcount = 1;
                     if (val->obj->garbage != NULL) val->obj->garbage(val, 0);
@@ -143,11 +145,11 @@ FAST_CALL void val_replace(Obj **val, Obj *val2) {
     *val = val_reference(val2);
 }
 
-size_t val_print(Obj *v1, FILE *f) {
+size_t val_print(Obj *v1, FILE *f, size_t max) {
     size_t len;
-    Obj *err = v1->obj->repr(v1, NULL, SIZE_MAX);
+    Obj *err = v1->obj->repr(v1, NULL, max);
     if (err == NULL) return 0;
-    if (err->obj == STR_OBJ) len = printable_print2(((Str *)err)->data, f, ((Str *)err)->len);
+    if (err->obj == STR_OBJ) len = printable_print2(Str(err)->data, f, Str(err)->len);
     else len = printable_print2((const uint8_t *)err->obj->name, f, strlen(err->obj->name));
     val_destroy(err);
     return len;
@@ -172,9 +174,9 @@ void destroy_values(void)
         size_t size = j * ALIGN;
         for (vals = slotcoll[j]; vals; vals = vals->next) {
             Obj *val = (Obj *)(vals + 1);
-            for (i = 0; i < SLOTS; i++, val = ((void *)val) + size) {
+            for (i = 0; i < SLOTS; i++, val = (Obj *)(((char *)val) + size)) {
                 if (val->obj != NULL) {
-                    val_print(val, stderr);
+                    val_print(val, stderr, SIZE_MAX);
                     fprintf(stderr, " %s %" PRIuSIZE " %" PRIxPTR "\n", val->obj->name, val->refcount, (uintptr_t)val);
                 }
             }
@@ -185,10 +187,12 @@ void destroy_values(void)
 
     for (j = 0; j < lenof(slotcoll); j++) {
         Slotcoll *s = slotcoll[j];
-        while (s != NULL) {
+        if (s == NULL) continue;
+        slotcoll[j] = NULL;
+        do {
             Slotcoll *old = s;
             s = s->next;
             free(old);
-        }
+        } while (s != NULL);
     }
 }
